// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/kernel/BKernel.java,v 1.21 1999/10/15 14:21:52 tp079519 Exp $

package au.edu.swin.synergy.kernel;

import java.io.FileReader;
import java.io.FileInputStream;
import java.io.LineNumberReader;
import java.lang.String;
import java.lang.Class;
import java.lang.Object;
import java.util.Vector;
import java.util.Hashtable;

import au.edu.swin.synergy.kernel.event.BBootCE;
import au.edu.swin.synergy.kernel.event.BShutdownCE;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.lib.BInputReader;

import java.io.Serializable;
import java.io.IOException;

/**
 * Synergy Micro-Kernel
 *
 * @Author John Newbigin
 * @Copyright Synergy Software 1999
 * @see BCommandEvent, BPlugin
 * @concurrency single instance, multiple threads
 * @rref Tristan's book
 */

public class BKernel implements Serializable
{
    /**
	 * If verbose is true then load messages are printed to System.out
	 */
    static final boolean verbose = true;
    
    
    /**
	 * A list of loaded module names
	 * This is used to prevent plugins from being loaded many times
	 */
    private BStringList moduleList = new BStringList();

	/**
	 * A list of modules which have dependancies which are being processed
	 * This is used to prevent circular dependancies
	 */
	private BStringList dependantModuleList = new BStringList();

	/**
	 * A list of modules which have faile to load
	 * do not try to load this again
	 */
	private BStringList failedModuleList = new BStringList();

	/**
	 * This is so we can get a refrence to a plugin by name
	 */

	private Hashtable pluginList = new Hashtable();
    
    /**
     * The registered listeners
     * This will possibly become a number of lists, one for each event type
     */
    private Vector listeners = null;
    
    /**
     * Creates a new BKernel
     * @post the configuration is completed and and plugins loaded
     * @param files a list of config files
     */
    
    
    public BKernel( BStringList files )
    {
		Debug.doPrintln("BKA: Create kernel " + files);
        listeners = new Vector();
        // load the list of modules...
        for( int i = 0; i < files.getCount(); i++ )
        {
            if( verbose )
            {
                System.out.println( "From " + files.get( i ) );
            }
            processConfFile( files.get( i ) );
        }
        
    }
    
    /**
     * Get a humam readable list of the loaded plugins
     * @pre the kernel is running
     * @return a list of the successfuly loaded modules
     */
    public BStringList getModuleList()
    {
        return moduleList;
    }
    
    /**
     * Register a new plugin with the kernel
     * @post the plugin is added to the list of command event listeners
     * @pre the kernel is running
     * @param plugin the plugin to register
     */
    
    public void doRegister( BPlugin plugin )
    {
		Debug.doPrintln("BKA: Registering plugin " + plugin);
        // add to a list...
		synchronized(listeners)
		{
	        listeners.add( plugin );
		}
    }
    
    /**
     * Un-register a plugin
     * @post the plugin is removed from the list of command event listeners
     * @pre the plugin is registered 
     * @param plugin the plugin to unregister
     */
    
    public void doUnRegister( BPlugin plugin )
    {
		Debug.doPrintln("BKA: Un-Registering plugin " + plugin);
        // add to a list...
		synchronized(listeners)
		{
	        listeners.remove( plugin );
		}
    }
	
    /**
     * post an event to the kernel
     * @post the event is being dispatched
     * @pre the kernel is running
     * @param sopurce the plugin sending the event
	 * @param event the event being dispatched
     * @bugs Can we get rid of the source and use event.source?
     */
    
    public void doDispatchEvent( Object source, BCommandEvent event )
    {
		try
		{
	        Debug.doPrintln( "BKA: Dispatching event " + event + " from " + source + " to " + listeners.size() + " plugins" );
	        // send the event to all the registered listeners
			synchronized(listeners)
			{
				event.setThreadCount(listeners.size());

	    	    for( int i = 0; i < listeners.size(); i++ )
		        {
	        	    BEventThread thread = new BEventThread( source, ( BPlugin ) listeners.get( i ), event );
	    	        thread.start();
		        }
			}
		}
		catch(Exception e)
		{
			Debug.doPrintln("BKA: Exception while dispatching event", e);
		}
    }

	
    /**
     * Shutdown a running kernel
     * @post a kernel shutdown is in progress
     * @pre the kernel is running
     */
    public void doShutdown()
    {
        Debug.doPrintln( "BKA: starting shutdown" );
        BShutdownCE event = new BShutdownCE( this );
		event.setCompletionObject(new OnShutdownComplete());
        doDispatchEvent( this, event );
        
        // we need to wait for something...?
        //System.exit( 0 );
    }

	private class OnShutdownComplete extends BCommandComplete
	{
		public void doComplete(BCommandEvent event, int status)
		{
			if(status == BCommandComplete.kDefault)
			{
				// no one objects....
				Debug.doPrintln("Nobody objects to a shutdown");
				System.exit(0);
			}
			if(status == BCommandComplete.kSuccess)
			{
				// see what they reckon
				BShutdownCE shutdownEvent = (BShutdownCE) event;
				if(shutdownEvent.getCancel())
				{
					Debug.doPrintln("Someone has objected");
					// not shutting down
				}
			}
		}
										
	}

	private boolean pluginLoaded( String name)
	{
		// see if the plugin is already loaded...
		synchronized(moduleList)
		{
			for(int i=0; i<moduleList.getCount(); i++)
			{
				if(moduleList.get(i).equals(name))
				{
					return true;
				}
			}
		}
		return false;
	}

	private boolean pluginFailed( String name)
	{
		// see if the plugin is already failed...
		synchronized(failedModuleList)
		{
			for(int i=0; i<failedModuleList.getCount(); i++)
			{
				if(failedModuleList.get(i).equals(name))
				{
					return true;
				}
			}
		}
		return false;
	}
	
    /**
     * Load the module from disk and create an instance
     * you can use the constructor, but not for messages
     * Once created, the kernel calls setKernel and then setup
     * you can use setup to send messages to register your UI.
     * @post the plugin is loaded
     * @pre the plugin is not already loaded
     * @return the new plugin object
     * @param the class name of the plugin, such as au.edu.swin.synergy.plugin.mod13.BWindowManager
     */
    public BPlugin loadModule( BStringList params, boolean autoboot )
    {
		try
		{
			String name = params.get(0);
	        Debug.doPrintln( "BKA: Loading plugin " + name );
			if(pluginLoaded(name))
			{
				Debug.doPrintln("BKA: Plugin " + name + " is already loaded");
				return null;
			}
			else if(pluginFailed(name))
			{
				Debug.doPrintln("BKA: Plugin " + name + " failed to load");
				return null;
			}
	
	        try
	        {
	            if( verbose )
	            {
	                System.out.print( name );
	            }
	            Class newClass = getClass().getClassLoader().loadClass( name );
	            if( verbose )
	            {
	                System.out.print( " ->" );
	            }
	            Object o = newClass.newInstance();
	            BPlugin plugin = ( BPlugin ) o;
	            plugin.setKernel( this );
	            
	            // look for the dependancies
				dependantModuleList.add(name);
	
				BStringList dependancies = plugin.getDependancies();
	            if( dependancies != null )
	            {
					// if any of the dependancies are in the dependantModuleList then we have a circular dependancy
					for(int i=0; i<dependancies.getCount(); i++)
					{
						if(!pluginLoaded(dependancies.get(i)))
						{
							loadModule(new BStringList(dependancies.get(i)), autoboot);
						}
						else
						{
							Debug.doPrintln("BKA: dependent plugin already loaded");
						}
					}
					
	                // we need to make sure that
	                // the dependancies are loaded
	                // else the dependancies don't list this module as a dependancy
	            }
	            
	            plugin.setup(params);
	            if( verbose )
	            {
	                System.out.println( " OK" );
	            }
	            Debug.doPrintln( "BKA: Plugin loaded" );
	            moduleList.add( name );
				pluginList.put( name, plugin);
	
				if(autoboot)
				{
					bootPlugin(plugin);
				}
	            return plugin;
	        }
	        catch( Exception E )
	        {
	            if( verbose )
	            {
	                System.out.println( " Fail" );
	            }
	            Debug.doPrintln( "BKA: Plugin load failed: " , E );
				failedModuleList.add(name);
	            return null;
	        }
		}
		catch(Exception E)
		{
			Debug.doPrintln("BKA: The plugin could not be loaded", E);
			return null;
		}
	}
	
    /**
     * UnLoad a currenlty loaded module
     * @post the plugin is unloaded
     * @pre the plugin is already loaded
     * @param the class name of the plugin, such as au.edu.swin.synergy.plugin.mod13.BWindowManager
     */
    public void unloadModule(String pluginName)
    {
		try
		{
			Debug.doPrintln("BKA: Unloading plugin " + pluginName);
			Object object = pluginList.get(pluginName);
			if(object != null)
			{
				BPlugin plugin = (BPlugin) object;
				plugin.unload();
				doUnRegister(plugin);
				pluginList.remove(pluginName);
				moduleList.remove(pluginName);
			}
			else
			{
				Debug.doPrintln("BKA: Plugin is not loaded!");
			}
		}
		catch(Exception E)
		{
			Debug.doPrintln("BKA: The plugin could not be unloaded", E);
		}
    }

	
    /**
     * Send the boot event to all loaded plugins
     * @post the kernel is booted
     */
    public void boot()
    {
		try
		{
	        if( verbose )
	        {
	            System.out.println( "Booting..." );
	            System.out.println( "" );
	        }
	        Debug.doPrintln( "BKA: Booting the loaded plugins" );
	        // start running....?
	        // we send a boot event, for test purposes at the moment
	        BBootCE event = new BBootCE( this );
	        doDispatchEvent( this, event );
		}
		catch(Exception e)
		{
			Debug.doPrintln("BKA: An error occured while booting the kernel", e);
		}
    }

	
    /**
     * Send a boot event to a single plugin
     * @param plugin The newly loaded plugin to be booted
     * @pre the kernel is running
     */
    
    public void bootPlugin( BPlugin plugin )
    {
		try
		{
	        if( verbose )
	        {
	            System.out.println( "Booting..." );
	            System.out.println( "" );
	        }
	        Debug.doPrintln( "BKA: Booting single plugin" );
	        // start running....?
	        // we send a boot event, for test purposes at the moment
	        BBootCE event = new BBootCE( this );
	        // send a boot to a specific module (loaded at run time...)
	        BEventThread thread = new BEventThread( this, plugin, event );
	        thread.start();
		}
		catch(Exception E)
		{
			Debug.doPrintln("BKA: An error occured while booting the plugin", E);
		}
    }

	
    /**
     * Process a kernel configuration file
     * @post the commands in the configuration file have been executed
     * @pre the kernel is not running
     * @return true if the the processing was successful
     * @param name the Name of the configuration file
     */
    private boolean processConfFile( String name )
    {
		Debug.doPrintln("BKA: processing " + name);
        try
        {
//            FileReader fileReader = new FileReader( name );
//            LineNumberReader modules = new LineNumberReader( fileReader );

			// first, try and open the file
            BInputReader modules = null;
			FileInputStream fileInputStream = null;
			
			try
			{
				fileInputStream = new FileInputStream(name);
				modules = new BInputReader( fileInputStream );
			}
			catch(Exception e)
			{
	            modules = new BInputReader( getClass().getResourceAsStream("/au/edu/swin/synergy/" + name) );
				if(modules == null)
				{
					System.out.println("Unable to load configuration file " + name);	
					return false;
				}
			}
            
            while( true )
            {
                String line = modules.readLine();
                if( line == null )
                {
                    break;
                }
                
                // we need to strip and comment
                int commentStart = line.indexOf( '#' );
                if( commentStart >= 0 )
                {
                    line = line.substring( 0, commentStart );
                }
                line = line.trim();

                
                if( line.length() > 0 )
                {
					BStringList params = new BStringList(line);
                    loadModule( params , false);
                }
            }
			if(fileInputStream != null)
			{
				fileInputStream.close();
			}
            return true;
        }
        catch( Exception E )
        {
            Debug.doPrintln( "BKA: Error processing " + name + ": " + E );
            return false;
        }
    }
}
