// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/kernel/BPlugin.java,v 1.14 1999/10/11 03:14:33 tp079519 Exp $

package au.edu.swin.synergy.kernel;

import java.lang.String;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Assertion;

/**
 * This is the plugin base class
 * @see BKernel, BCommandEvent
 * @concurrency each plugin creates one plugin, multiple threads
 * @copyright Synergy Software 1999
 * @rref Tristan's Book
 * @author John Newbigin
 */

public abstract class BPlugin
{
	/**
	 * The plugin retains a refrence to the kernel so it can dispatch events.
	 */
    protected BKernel kernel;
    
	/**
	 * This method needs to be implemented by all plugins.  When the kernel dispatched an event
	 * it is passed in to this method.  
	 * <p>
	 * A new thread is created for each event and plugin, so it is possible to have blocking code in this method.
     * @throws any exception thrown by this method is ignored.
     * @param source a refrence to the plugin sending the event.
	 * @param event the event to process
	 */
    public abstract void onBCommandEvent( Object source, BCommandEvent event );
	
    /**
     * Return a refrence to the connected kernel
     * @pre the kernel has been set using the setKernel method
     * @return the connected kernel
     */
    
    public BKernel getKernel()
    {
        return kernel;
    }
    
    
    /**
     * Store a refrence to the kernel, and register the plugin.
	 * This is called by the kernel before it calls the setup method.
	 * A sub-class could override this method if it did not want to become
	 * registered to receive events
     * @post the plugin is registered with the kernel
     * @pre the plugin is not registered
     * @param fKernel the kernel object
     */
    
    public void setKernel( BKernel fKernel )
    {
        kernel = fKernel;
        kernel.doRegister( this );
    }

	
    /**
     * Return a list of plugin dependancies
	 * This is currently not used by the kernel
     * @return a list of plugins such as "au.edu.swin.synergy.plugin.mod13.BWindowManager"
	 * or null if there are no dependancies
     */
    
    public BStringList getDependancies()
    {
		// we return null, because by default, we do not do dependancies
		// sub-classes can override the method to specify there own dependancies
        return null;
    }
	
    /**
     * this method is called by the kernel after the call to setKernel.  A Plugin should use override
	 * this method rather than having a constructor.
	 * It is safe to dispatch events in this method, but be aware that the destination plugin
	 * will not have reveived the boot event.
     * @post plugin specific setup is complete
     * @pre the kernel has been set
	 * @deprecated as of BKA build-5.  The kernel now only calls <code> setup(BStringList param)</code>
     */
    
    public final void setup()
    {
    }

    /**
     * this method is called by the kernel after the call to setKernel.  A Plugin should use override
	 * this method rather than having a constructor.
	 * It is safe to dispatch events in this method, but be aware that the destination plugin
	 * will not have reveived the boot event.
     * @post plugin specific setup is complete
     * @pre the kernel has been set
	 * @param a list of command line parameters param.get(0) is the name of the plugin
     */
    public void setup(BStringList param)
    {
    }

    
    /**
     * Request that a plugin unregister it's self
     * after the call to unload, the plugin will be unregistered
     * so it no longer gets any events.
     * @post the plugin has removed all refrences to it's components (such as from the window manager, command line etc.
     * @pre the plugin has been setup
     * @throws none
     * @return void
     */

	public void unload()
	{
	}

    /**
     * DispatchEvent
     * @post the event has been dispatched
     * @pre the plugin is registered
     * @throws none
     * @return void
     * @param an event
     */
    
    public void dispatchEvent( BCommandEvent event )
    {
        kernel.doDispatchEvent( this, event );
    }
    
    /**
     * DispatchEventServer
     * @post the event has been dispatched, and will be redirected to the server
     * @pre the plugin is registered
     * @throws none
     * @return void
     * @param an event
     */
    public void dispatchEventServer( BCommandEvent event )
    {
        event.setNetworkStatus( BCommandEvent.kServer );
        kernel.doDispatchEvent( this, event );
    }
    
    /**
     * DispatchEventTo
     * @post the event has been dispatched and will be redirected to the specified client
     * @pre the plugin is registered
     * @throws none
     * @return void
     * @param an event
     */
    
    public void dispatchEventTo( BCommandEvent event, int fConnectionID )
    {
        event.setNetworkStatus( BCommandEvent.kClient );
        event.setConnectionID( fConnectionID );
        kernel.doDispatchEvent( this, event );
    }

    /**
     * Dispatch an event and wait for completion before returning
     * @post The completion of the event has been called
     * @pre the event does not contain a completion object
     * @throws none
     * @return the blocking completion object.  Use this object to extract the event and completion status.
     * @param event the event to dispatch.  This event should not have a completion object.
     */
    
    public BBlockingComplete dispatchEventAndWait( BCommandEvent event )
    {
		Assertion.doAssert(event.getCompletionObject() == null, "You can not specify a completion object when using dispatchAndWait");

		// check if we are in the AWT thread...
		Assertion.doAssert(Thread.currentThread().getName().indexOf("AWT") == -1, "You can not call dispatchAndWait from the AWT thread");

		BBlockingComplete complete = new BBlockingComplete();
		event.setCompletionObject(complete);
		
        dispatchEvent( event );

		complete.waitForEvent();
		return complete;
    }


    /**
     * Dispatch event to the server and wait for completion before returning
     * @post The completion of the event has been called
     * @pre the event does not contain a completion object
     * @throws none
     * @return the blocking completion object.  Use this object to extract the event and completion status.
     * @param event the event to dispatch.  This event should not have a completion object.
     */
    
    public BBlockingComplete dispatchEventServerAndWait( BCommandEvent event )
    {
		Assertion.doAssert(event.getCompletionObject() == null, "You can not specify a completion object when using dispatchAndWait");
		// check if we are in the AWT thread...
		Assertion.doAssert(Thread.currentThread().getName().indexOf("AWT") == -1, "You can not call dispatchAndWait from the AWT thread");

		BBlockingComplete complete = new BBlockingComplete();
		event.setCompletionObject(complete);
		
        dispatchEventServer( event );

		complete.waitForEvent();
		return complete;
    }

    /**
     * Dispatch event to the specified client and wait for completion before returning
     * @post The completion of the event has been called
     * @pre the event does not contain a completion object
     * @throws none
     * @return the blocking completion object.  Use this object to extract the event and completion status.
     * @param event the event to dispatch.  This event should not have a completion object.
     */
    
    public BBlockingComplete dispatchEventToAndWait( BCommandEvent event, int fConnectionID )
    {
		Assertion.doAssert(event.getCompletionObject() == null, "You can not specify a completion object when using dispatchAndWait");

		BBlockingComplete complete = new BBlockingComplete();
		event.setCompletionObject(complete);
		
        dispatchEventTo( event, fConnectionID );

		complete.waitForEvent();
		return complete;
    }
}
