// $Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/lib/BFile.java,v 1.10 1999/08/28 10:20:17 tp079519 Exp $
package au.edu.swin.synergy.lib;

import java.util.zip.*;
import java.io.*;
import java.lang.*;
import javax.swing.JOptionPane;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Debug;


/**
 * Encapsulates a file(s) which may be sent accross the network.
 * File name(s) are passed through to the constructor which then
 * opens and compresses to allow minimal tramit delay over network.
 * When this class is received on the other end it is to be
 * decompressed in working directory.
 *
 * @author Christian Karolos
 * @bugs If file names include path information, it will be assumed those
 *       paths will exist on recieving machine.
 * @copyright Copyright (c) 1999
 */

public class BFile implements Serializable
{
    public final static int kMinCompressionRate = 0;
    public final static int kMaxCompressionRate = 9;
    public final static int kDefaultCompressionRate = 5;
    
    /**
     * Name of file to be compressed and sent.
     */
    File file;
    
    /**
     * Names of files to be compressed and sent.
     */
    File [] fileGroup;
    
    /**
     * Specifies whether one or many files are to be compressed.
     */
    boolean manyFiles;
    
    /**
     * Specifies whether file(s) registered with class have been compressed.
     */
    boolean fileCompressed;
    
    /**
     * Compression rate specified by the user.
     */
    int compressionRate;
    
    /**
     * Contains compressed file(s).
     */
    byte [] fileBuffer;
    
    
    /**
     * Constructor which registers the file to be compressed at a
     * specified rate.
     *
     * @param fileName Name of file to be compressed
     * @param compressionRate Rate at which file is to be compressed.
     * Please complete the missing tags for BFile
     */
    public BFile( File file, int compressionRate )
    {
        this.file = file;
        this.manyFiles = false;
        this.fileCompressed = false;
        this.setCompressionRate( compressionRate );
    }
    
    /**
     * Constructor which registers files to be compressed at a
     * specified rate.
     *
     * @param fileGroup Group of files to be compressed
     * @param compressionRate Rate at which file is to be compressed.
     */
    public BFile( File [] fileGroup, int compressionRate )
    {
        this.fileGroup = fileGroup;
        this.manyFiles = true;
        this.fileCompressed = false;
        this.setCompressionRate( compressionRate );
    }

    /**
     * Returns file names of all file compressed in this object.
     * @return List of file names.
     * @pre Files have been compressed and set.
     */    
    public BStringList getFileNames()
    {
        // This can be left until later on.
        
        BStringList list = new BStringList();
        
        // Retrieve and insert file names into list if this BFile
        // contains many compressed files.
        
        return list;
    }

    /**
     * If only one file has been compressed, return it's file name.
     * @return Name of compressed file.
     * @pre File has been set.
     */    
    public String getFileName()
    {
        return file.getName();
    }
    
    /**
     * Sets the rate at which file(s) are to be compressed.
     *
     * @param rate User specified compression rate.
     *
     * @modifies compressionRate
     */
    public void setCompressionRate( int rate )
    {
        // Save the hassle of handling a thrown exception later on.
        if( rate > kMinCompressionRate && rate < kMaxCompressionRate )
        {
            this.compressionRate = rate;
        }
        else
        {
            this.compressionRate = this.kDefaultCompressionRate;
        }
    }

    /**
     * Signifies whether multiple files have been compressed.
     * @return True if many files have been compressed.
     */    
    public boolean multipleFiles()
    {
        return manyFiles;
    }
    
    /**
     * Compresses file(s) specified when this object was created. Then writes to our
     * file buffer whatever has been compressed.
     *
     * @modifies fileBuffer
     * @modifies fileCompressed
     *
     * @return Boolean indicating whether the compression process went to plan
     *
     * @throws IOException
	 *
	 * @pre File name has been set.
	 * @post File compressed.
     */
    public boolean compress()
    {
        boolean nothingWentWrong = true;
        try
        {
            ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
            ZipOutputStream zipOut = new ZipOutputStream( byteOut );
            zipOut.setMethod( ZipOutputStream.DEFLATED );
            zipOut.setLevel( this.compressionRate );
            
            if( this.manyFiles )
            {
                // For every file...
                for( int i = 0; i < fileGroup.length && nothingWentWrong; i++ )
                {
                    nothingWentWrong = writeToOutput( zipOut, this.fileGroup [ i ] );
                }
            }
            else
            {
                nothingWentWrong = writeToOutput( zipOut, file );
            }
            
            // Till get it right change so writes to file.
            this.fileBuffer = byteOut.toByteArray();
            
            zipOut.close();
            byteOut.close();
        }
        catch( IOException ioe )
        {
            Debug.doPrintln( ioe.getMessage() );
            nothingWentWrong = false;
        }
        
        if( nothingWentWrong )
        {
            this.fileCompressed = true;
        }
        
        return nothingWentWrong;
    }
    
    /**
     * Mainly for testing purposes will write whatever is contained in our buffer
     * to file of users choice.
     *
     * @pre File(s) must have been compressed
     *
     * @param fName Name of file compressed data is written to
     *
     * @return Boolean signifing is zip successfully created
     *
     * @throws IOException
     * @post File compressed and file created.
     */
    public boolean createZipFile( String fName )
    {
        boolean zipCreated = true;
        
        if( this.fileCompressed )
        {
            try
            {
                File f = new File( fName );
                FileOutputStream fout = new FileOutputStream( f );
                
                fout.write( this.fileBuffer, 0, this.fileBuffer.length );
                
                fout.close();
            }
            catch( IOException ioe )
            {
                Debug.doPrintln( ioe.getMessage() );
                zipCreated = false;
            }
        }
        else
        {
            Debug.doPrintln( "Compression has not taken place, must be done before\ndata can be written to file" );
            zipCreated = false;
        }
        
        return zipCreated;
    }
    
    /**
     * Expands file(s) registered with this class to working directory.
     *
     * @pre CompressFile () must have been called
     *
     * @throws FileNotFoundException
     * @throws IOException
     *
     * @return Boolean indicating success of decompression
     * @post File expanded in working directory.
     */
    public boolean expand()
    {
        return this.expand( null );
    }
    
    /**
     * Expands file(s) registered with this class into directory of users choice.
     *
     * @pre CompressFile () must have been called
     *
     * @throws FileNotFoundException
     * @throws IOException
     *
     * @return Boolean indicating success of decompression
     * @param f File path to save file.
     * @post File expanded in path of users choice.
     */
    public boolean expand( File f )
    {
        boolean everythingExpanded = true;
        
        if( this.fileCompressed )
        {
            try
            {
                int b;
                ZipEntry zEntry;
                
                // Setting up input stream where file will be read.
                ByteArrayInputStream byteIn = new ByteArrayInputStream( this.fileBuffer );
                ZipInputStream zipIn = new ZipInputStream( byteIn );
                
                FileOutputStream fOut;
                File test;
                
                // For every zip entry registered create file and write to output.
                for( int i = 0; ( zEntry = zipIn.getNextEntry() ) != null; i++ )
                {
                    if( f == null )
                    {
                        fOut = new FileOutputStream( zEntry.getName() );
                    }
                    else
                    {
                        fOut = new FileOutputStream( f );
                    }
                    
                    while( ( b = zipIn.read() ) != - 1 )
                    {
                        fOut.write( b );
                    }
                    
                    zipIn.closeEntry();
                    fOut.close();
                    
                    //test = new File( zEntry.getName() );
                    if( f/*test*/.exists() )
                    {
                        Debug.doPrintln( "Expanded file exists in " + f.getPath());
                    }
                    else
                    {
                        Debug.doPrintln( "Expanded file does not exist" );
                    }
                }
                
                zipIn.close();
                byteIn.close();
            }
            catch( FileNotFoundException fnf )
            {
                Debug.doPrintln( fnf.getMessage() );
                everythingExpanded = false;
            }
            catch( IOException ioe )
            {
                Debug.doPrintln( ioe.getMessage() );
                everythingExpanded = false;
            }
        }
        else
        {
            Debug.doPrintln( "Cannot uncompress file which hasn't been compressed." );
        }
        
        return everythingExpanded;
    }
    
    /**
     * Opens the file passed and reads every byte into a zip output sream
     * where the compressing is done.
     *
     * @param zipOut Zip output stream where bytes are written to
     * @param file Name of file to be opened and written to output stream
     *
     * @return Boolean indicating whether writing process was valid
     *
     * @throws FileNotFoundException
     * @throws IOException
     */
    private boolean writeToOutput( ZipOutputStream zipOut, File userFile )
    {
        boolean everythingWentToPlan = true;
        try
        {
            // Temp array holding bytes contained in "file".
            byte [] b;
            FileInputStream fileIn = new FileInputStream( userFile );
            
            b = new byte [ ( int ) userFile.length() ];
            
            // Creating a new "ZipEntry" of "file", needed when writing and reading
            // zip type files.
            ZipEntry ze = new ZipEntry( userFile.getName() );
            ze.setSize( userFile.length() );
            Debug.doPrintln( "Size in zip entry before compressing is : " + ze.getSize() );
            
            zipOut.putNextEntry( ze );
            
            fileIn.read( b, 0, ( int ) userFile.length() );
            zipOut.write( b, 0, ( int ) userFile.length() );
            
            zipOut.closeEntry();
            fileIn.close();
        }
        catch( FileNotFoundException fnf )
        {
            Debug.doPrintln( fnf.getMessage() );
            everythingWentToPlan = false;
        }
        catch( IOException ioe )
        {
            Debug.doPrintln( ioe.getMessage() );
            everythingWentToPlan = false;
        }
        
        return everythingWentToPlan;
    }
}
