//$

package au.edu.swin.synergy.lib.io;

import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.*;
import javax.swing.filechooser.*;

/**
 * An extension of the FileFilter class to allow filtering of only those
 * filetypes that are necessary to display in a JFileChooser
 *
 * @author Matthew Evans
 *
 */
public class BFileFilter extends FileFilter
{
   /**
    * The collection of filetype extensions to be filtered
    */   
    private Hashtable fileExtensions = null;
    
    /**
     * A description of a file filter
     */
    private String description = new String();

    /**
     * Creates a file filter. If no filters are added, then all
     * files are accepted.
     */
    public BFileFilter() 
    {
       this.fileExtensions = new Hashtable();
    }

    /**
     * Creates a file filter that accepts files with the given extension.
     * Example: new BFileFilter("jpg");
     *
     * @see #addExtension
     */
    public BFileFilter(String extension) 
    {
       this(extension, null);
    }

    /**
     * Creates a file filter that accepts the given file type, and
     * is supplied with an appropriate description.
     *
     * @param extension the extension to filter against
     * @param description a description of this file filter
     */
    public BFileFilter(String extension, String description) 
    {
      this();
      
      if(extension != null)
      {
          addExtension(extension);
      }
    
      if(description != null) 
      {
         this.description = description;
      }
    }


    /**
     * Returns whether this file should be displayed in a JFileChooser
     *           
     * @param file the file to accept or reject
     * @return true if the file was accepted; false otherwise
     */
    public boolean accept(File file) 
    {
      if(file != null) 
      {
       if(file.isDirectory()) 
       {
         return true;
       }
       
       String extension = getExtension(file);
       
       if(extension != null && fileExtensions.containsKey(getExtension(file))) 
       {
         return true;
       }
      }
      
      return false;
    }

    /**
     * Return only the extension part of the specified file
     *
     * @param the file to act upon
     * @return the extension part of the filename
     */
    public String getExtension(File file) 
    {
      if(file != null) 
      {
         String filename = file.getName();
         int i = filename.lastIndexOf('.');
         
         if(i > 0 && i < filename.length() - 1) 
         {
            return filename.substring(i+1).toLowerCase();
         }
      }
      
      return null;
    }

    /**
     * Adds an extension for filtering. eg. "tit".
     *
     * @param extension the filetype extension to add to the file filter
     */
    public void addExtension(String extension)
    {
      if(fileExtensions == null)
      {
         fileExtensions = new Hashtable();
      }

      fileExtensions.put(extension.toLowerCase(), this);
    }


    /**
     * Returns a description of this file filter. 
     * Eg. "Cohesion Project files (*.cop)"
     *
     * @return a description of the file filter
     */
    public String getDescription()
    {
       Enumeration extensions = fileExtensions.keys();
       String tempDescription = new String(description);
       tempDescription += " (";
       
       if(extensions != null)
       {
          if (extensions.hasMoreElements())
          {
              tempDescription += "." + (String) extensions.nextElement();

             while (extensions.hasMoreElements())
              {
                 tempDescription += ", " + (String) extensions.nextElement();
              }
          }

          tempDescription += ")";
       
          return tempDescription;
       } 
        else 
       {
         return description;
       }
    }

    /**
     * Sets the description of this filter. 
     * Eg. "Cohesion Project files"
     *
     * @param description a description of the filter
     */
    public void setDescription(String description) 
    {
       this.description = description;
    }

}