
/**
 * Utility class for saving 
 * BasicStrokes.
 *
 * @author Imron Alston
 * @copyright Team Synergy 1999
 *
 */

package au.edu.swin.synergy.lib.io;

import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;

import au.edu.swin.synergy.debug.Debug;

import java.util.Vector;
import java.awt.*;

public class BStrokeIO
{
	protected static final String kKeyStrokeWidth = "width";
	protected static final String kDefaultStrokeWidth = "1.0";

	protected static final String kKeyStrokeCap = "cap";
	protected static final String kDefaultStrokeCap = "" + BasicStroke.CAP_SQUARE;

	protected static final String kKeyStrokeJoin = "join";
	protected static final String kDefaultStrokeJoin = "" + BasicStroke.JOIN_MITER;

	protected static final String kKeyStrokeLimit = "limit";
	protected static final String kDefaultStrokeLimit = "1.0";

	protected static final String kKeyStrokePhase = "phase";
	protected static final String kDefaultStrokePhase = "0.0";

	protected static final String kKeyStrokeDash = "dash";

	public static boolean save( BDataFile file, BasicStroke stroke )
	{
		// the stroke...
		file.addData( kKeyStrokeWidth, "" + stroke.getLineWidth(), kDefaultStrokeWidth);
		file.addData( kKeyStrokeCap, "" + stroke.getEndCap(), kDefaultStrokeCap);
		file.addData( kKeyStrokeJoin, "" + stroke.getLineJoin(), kDefaultStrokeJoin);
		file.addData( kKeyStrokeLimit, "" + stroke.getMiterLimit(), kDefaultStrokeLimit);
		file.addData( kKeyStrokePhase, "" + stroke.getDashPhase(), kDefaultStrokePhase);

		float[] dash = stroke.getDashArray();
		if(dash != null)
		{
			for(int i=0; i< dash.length; i++)
			{
				Debug.doPrintln ( "\nSaving : " + Float.toString ( dash[i] ) );
				file.addData( kKeyStrokeDash, Float.toString ( dash[i]));
			}
		}

		return true;
	}

	/**
	 * Save a stroke, but only what is different to the defaults
	 */
	public static boolean save( BDataFile file, BasicStroke stroke, BasicStroke defaults )
	{
		// the stroke...
		file.addData( kKeyStrokeWidth, "" + stroke.getLineWidth(), "" + defaults.getLineWidth());
		file.addData( kKeyStrokeCap, "" + stroke.getEndCap(), "" + defaults.getEndCap());
		file.addData( kKeyStrokeJoin, "" + stroke.getLineJoin(), "" + defaults.getLineJoin());
		file.addData( kKeyStrokeLimit, "" + stroke.getMiterLimit(), "" + defaults.getMiterLimit());
		file.addData( kKeyStrokePhase, "" + stroke.getDashPhase(), "" + defaults.getDashPhase());

		// if the dash is different, we will save it

		float[] dash = stroke.getDashArray();
		float[] defaultDash = defaults.getDashArray();
		
		boolean different = false;
		
		if(dash != null && defaultDash == null)
		{
			different = true;
		}
		else
		{
			if(defaultDash != null && dash != null)
			{
				if(dash.length != defaultDash.length)
				{
					different = true;
				}
				else
				{
					for(int i=0; i<dash.length; i++)
					{
						if(dash[i] != defaultDash[i])
						{
							different = true;
							break;
						}
					}
				}
			}
		}
		if(different && dash != null)
		{
			for(int i=0; i< dash.length; i++)
			{
				Debug.doPrintln ( "\nSaving : " + Float.toString ( dash[i] ) );
				file.addData( kKeyStrokeDash, Float.toString ( dash[i]));
			}
		}

		return true;
	}
	
	public static BasicStroke load( BDataFile file )
	{
		BasicStroke result = null;

		// load the stroke bits...
		float width = Float.parseFloat(file.getData(kKeyStrokeWidth, kDefaultStrokeWidth));
		int cap = Integer.parseInt(file.getData(kKeyStrokeCap, kDefaultStrokeCap));
		int join = Integer.parseInt(file.getData(kKeyStrokeJoin, kDefaultStrokeJoin));
		float limit = Float.parseFloat(file.getData(kKeyStrokeLimit, kDefaultStrokeLimit));
		float phase = Float.parseFloat(file.getData(kKeyStrokePhase, kDefaultStrokePhase));

		Vector dashVec = new Vector();
        while ( true )
        {
			String value = file.getData(kKeyStrokeDash, null);
			Debug.doPrintln ( "\nReading dash : " + value );
			if(value == null)
			{
				break;
			}
            dashVec.add (new Float( value ) );
        }

        float [] dashes = null;
		
		if(dashVec.size () > 0)
		{
			dashes = new float[ dashVec.size () ];
	        for ( int i = 0; i < dashVec.size (); i++ )
	        {
		        dashes[ i ] = ( ( Float ) dashVec.elementAt ( i ) ).floatValue ();
				Debug.doPrintln("Dash = " + dashes[ i ]); 
	        }
		}
        result = new BasicStroke ( width, cap, join, limit, dashes, phase );

		return result;
	}

	public static BStrokeIO loadDefault( BDataFile file )
	{
		BStrokeIO result = new BStrokeIO();

		// load the stroke bits...
		result.width = file.getData(kKeyStrokeWidth, null);
		result.cap = file.getData(kKeyStrokeCap, null);
		result.join = file.getData(kKeyStrokeJoin, null);
		result.limit = file.getData(kKeyStrokeLimit, null);
		result.phase = file.getData(kKeyStrokePhase, null);

		Vector dashVec = new Vector();
        while ( true )
        {
			String value = file.getData(kKeyStrokeDash, null);
			Debug.doPrintln ( "\nReading dash : " + value );
			if(value == null)
			{
				break;
			}
            dashVec.add (new Float( value ) );
        }

		if(dashVec.size() > 0)
		{
			result.dashes = new float[ dashVec.size () ];
	        for ( int i = 0; i < dashVec.size (); i++ )
	        {
		        result.dashes[ i ] = ( ( Float ) dashVec.elementAt ( i ) ).floatValue ();
				Debug.doPrintln("Dash = " + result.dashes[ i ]); 
	        }
		}

		return result;
	}

	private String width = null;
	private String cap = null;
	private String join = null;
	private String limit = null;
	private String phase = null;
	private float []dashes = null;

	public BasicStroke getStroke(BasicStroke stroke)
	{
		BasicStroke result = null;
		float swidth = (width == null ? stroke.getLineWidth() : Float.parseFloat(width));
		int scap = (cap == null ? stroke.getEndCap() : Integer.parseInt(cap));
		int sjoin = (join == null ? stroke.getLineJoin() : Integer.parseInt(join));
		float slimit = (limit == null ? stroke.getMiterLimit() : Float.parseFloat(limit));
		float sphase = (phase == null ? stroke.getDashPhase() : Float.parseFloat(phase));

		float []sdashes = (dashes == null ? stroke.getDashArray() : dashes);

        result = new BasicStroke ( swidth, scap, sjoin, slimit, sdashes, sphase );

		return result;
	
	}
}
