//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/lib/io/BTextdb.java,v 1.6 1999/09/14 10:12:20 tp079210 Exp $

package au.edu.swin.synergy.lib.io;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * This class allows lines of text to be read and written to text files.
 * Comments starting with # are ignored
 * the lines are stripped of leading and trailing white space
 * blank lines are skipped
 * and the resulting lines are returned in the form of a BTextdbRecord
 *
 * see sample test class for details
 *
 * @author John Newbigin
 * @see BTextdbRecord
 * @concurrency NA
 * @copyright Synergy Software
 * @rref
 */

public class BTextdb
{
    private String fileName;
    private InputStream inputStream = null;
    private OutputStream outputStream = null;
    
    private boolean eof;

	
    /**
     * Creates a new database object
     * @param fName the name of the file
     */
    
    public BTextdb( String fName )
    {
        fileName = fName;
    }

    /**
     * Creates a new text db with no file name
     * specified. This assumes that the user will
     * be supplying their own input or output streams.
     */
    public BTextdb()
    {
    }

	
    /**
     * Get a record from the database
     * @pre the file has been opened for reading
     * @return the record from the database, or null if there are no more records
     */
    
    public BTextdbRecord getRecord()
    {
        String str;
        
        while( true )
        {
            str = getLine();
            if( str == null )
            {
                return null;
            }
            
            int comment = str.indexOf( '#' );
            if( comment >= 0 )
            {
                str = str.substring( 0, comment );
            }
            str = str.trim();
            if( str.length() > 0 )
            {
                break;
            }
        }
        
        BTextdbRecord record = new BTextdbRecord();
        record.setRecord( str );
        return record;
    }

	
    /**
     * Write a record to the database
     * @pre the file has been opened for writing
     * @param record the record to add
     */
    
    public void addRecord( BTextdbRecord record )
    {
        try
        {
            outputStream.write( record.getRecord().getBytes() );
            outputStream.write( "\r\n" .getBytes() );
        }
        catch( IOException e )
        {
            // ?
        }
    }

	
    /**
     * openFileForRead
     * @post the file is opened for reading
     * @return true if the file could be opened, otherwise false
     */
    
    public boolean openFileForRead()
    {
        try
        {
            FileInputStream tempInputStream = new FileInputStream( fileName );
            return this.openFileForRead(tempInputStream);
        }
        catch( IOException e )
        {
            return false;
        }
    }

    /**
     * This will use the given input stream as the source of
     * the file to read from.
     *
     * @param   inputStream The input stream to read from
     */
    public boolean openFileForRead(InputStream newInputStream)
    {
        inputStream = newInputStream;
        eof = false;
        return true;
    }


    /**
     * opens a file for writing
     * @post the file is opened for writing
     * @return true if the file could be opened
     * @param append if append is false, then any existing data will be overwritten
     */

    public boolean openFileForWrite( boolean append )
    {
        try
        {
            FileOutputStream tempOutputStream = new FileOutputStream( fileName, append );
            return this.openFileForWrite(append, tempOutputStream);
        }
        catch( IOException e )
        {
            return false;
        }
    }

    /**
     * opens a file for writing
     * @post the file is opened for writing
     * @return true if the file could be opened
     * @param append if append is false, then any existing data will be overwritten
     */

    public boolean openFileForWrite( boolean append, OutputStream newOutputStream)
    {
        outputStream = newOutputStream;
        return true;
    }


    /**
     * close an open file
     * @post the file is closed
     * @pre the file is open
     */

    public void close()
    {
        try
        {
            if( inputStream != null )
            {
                inputStream.close();
                inputStream = null;
            }
            if( outputStream != null )
            {
                outputStream.close();
                outputStream = null;
            }
        }
        catch( IOException e )
        {
            // ?
        }
    }

	
    /**
     * get a line of text from the file
     * @post the file pointer is movel along one line
     * @pre the file is open for reading
     * @return the string, with no new line character, or if the end of the file is reached returns null
     */
    
    private String getLine()
    {
        if( eof )
        {
            return null;
        }
        
        String str = new String();
        while( true )
        {
            int letter;
            try
            {
                letter = inputStream.read();
            }
            catch( Exception e )
            {
                letter = - 1;
            }
            
            if( letter == - 1 )
            {
                eof = true;
                break;
            }
            else if( letter == 13 )
            {
                // ignore
            }
            else if( letter == 10 )
            {
                break;
            }
            else
            {
                str = str + ( char ) letter;
            }
            
        }
        return str;
    }
}

/*
class test
{
    public static void main(String[] args)
    {
        BTextdb db = new BTextdb("test.txt");
        db.openFileForRead();

        BTextdb db2 = new BTextdb("test2.txt");
        db2.openFileForWrite(false);

        while(true)
        {
            BTextdbRecord record = db.getRecord();
            if(record == null)
            {
                break;
            }

            db2.addRecord(record);
 
            System.out.println("Results:");
            for(int i=0; i<record.getCount(); i++)
            {
                System.out.println(record.getElement(i));
            }
            //System.out.println(record.getRecord());
        }
    }
}
*/
