//$
package au.edu.swin.synergy.plugin.addressbook;

import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.event.BBootCE;
import au.edu.swin.synergy.plugin.addressbook.event.BAddressBookCE;
import au.edu.swin.synergy.plugin.addressbook.event.BAddressDisplayCE;
import au.edu.swin.synergy.plugin.server.event.BGetNetworkStatusCE;
import au.edu.swin.synergy.lib.BStringList;

import au.edu.swin.synergy.plugin.help.BHelpItem;
import au.edu.swin.synergy.plugin.help.BHelpUtil;
import au.edu.swin.synergy.plugin.help.BHelpTreeModel;
import au.edu.swin.synergy.plugin.help.event.BRegisterHelpCE;

import java.util.Vector;
import java.io.*;

/**
 * Represents the manager of an address book on the client or the server.
 * BAddressBookMgr receives requests to process address book related reuests
 * (save address, update address, get address book, delete address), and 
 * subsequently completes the necessary task.
 *
 * @author Matthew Evans
 */
public class BAddressBookMgr extends BPlugin
{

   /**
    * The store of registered users
    */
   private BAddressBook addressBook = new BAddressBook();

   /**
    * The name of the file to store address book details to
    */
   private static String kClientBookFileName = new String("AddressBook.dat");
   private static String kServerBookFileName = new String("ServerAddressBook.dat");
   private String addressBookFileName = kClientBookFileName;
    

   /**
    * Identifies whether an instance of the BAddressBookMgr is on the server or
    * client.
    */
   private int addressBookLocation;
    
   /**
    * Creates the manager and initialises the address database
    */
   public void setup(BStringList params)
   {
      BGetNetworkStatusCE findStatus = new BGetNetworkStatusCE(this, new BGetNetworkStatusCC());
      dispatchEvent(findStatus);
      doRestoreAddressBook();
   }
    
   /**
    * When any data is sent across from a client, this method is called
    * to determine whether it is a request for Address Book data.
    *
    * @param  source The origin of the event
    * @param  event The event delegated
    */
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
		if ( event instanceof BBootCE )
		{
			registerHelp ();
		}
        else if( event instanceof BAddressBookCE )
        {
            Debug.doPrintln( "\nBAddressBookMgr: onCommandEv: received address book event" );
            
            if (this.addressBookLocation == BGetNetworkStatusCE.kServer && event.getNetworkStatus() == BCommandEvent.kServer)
				doProcessAddressBookEvent((BAddressBookCE) event);
			else if (this.addressBookLocation == BGetNetworkStatusCE.kClient && event.getNetworkStatus() == BCommandEvent.kClient)
				doProcessAddressBookEvent((BAddressBookCE) event);
        }
    }

	/**
	* Register out help with the help system.
	*/
	public void registerHelp ()
	{
		try
		{
			BHelpTreeModel tree = new BHelpTreeModel ( "Address Book" );
			String path = "/au/edu/swin/synergy/plugin/addressbook/help/";

			BHelpItem ae = new BHelpItem ( "Adding an Entry", path + "addAddress.html" );
			BHelpItem ee = new BHelpItem ( "Editing an Entry", path + "editAddress.html" );
			BHelpItem de = new BHelpItem ( "Deleting an Entry", path + "deleteAddress.html" );

			tree.addHelpItem ( null, ae );
			tree.addHelpItem ( null, ee );
			tree.addHelpItem ( null, de );

			BRegisterHelpCE event = new BRegisterHelpCE ( this, tree );
			dispatchEvent ( event );
		}
		catch ( Exception e )
		{
			e.printStackTrace ();
		}
	}

    /**
     * Handle the processing necessary on receipt of a BAddressBookCE object
     *
     * @param addressEvent the event received
     */
    private void doProcessAddressBookEvent(BAddressBookCE addressEvent)
    {
       int result = BCommandComplete.kError;
       
       if(addressEvent.getRequest() == BAddressBookCE.kListAddressBook)
       {
         //Send a list of addresses back
         addressEvent.setAddressList(this.doGetAddressBook());
         addressEvent.setAddressBookOrigin(this.addressBookLocation);
         result = BCommandComplete.kSuccess;
       }
       else if(addressEvent.getRequest() == BAddressBookCE.kAddAddress)
       {
         if (this.addressBook.addEntry(addressEvent.getAddress()))
         {
           this.addressBook.save(addressBookFileName);
           result = BCommandComplete.kSuccess;
         }
       }
       else if(addressEvent.getRequest() == BAddressBookCE.kRemoveAddress)
       {
         //Remove the address
         if(this.addressBook.removeEntry(addressEvent.getAddress()))
         {
            result = BCommandComplete.kSuccess;
            this.addressBook.save(addressBookFileName);
         }
         
       }
       else if(addressEvent.getRequest() == BAddressBookCE.kUpdateAddress)
       {
         //Edit the address
         if (this.addressBook.editEntry(addressEvent.getAddress(), addressEvent.getModifyAddress()))
         {
            result = BCommandComplete.kSuccess;
            this.addressBook.save(addressBookFileName);
         }   
         
       }
       
       Debug.doPrintln("BAddressBookMgr: location=" + addressBookLocation + "; processed a request (" + addressEvent.getRequest() + "); result=" + result);
       addressEvent.doComplete(result);
    }
    

   /**
    * Returns the addressBook
    *
    * @return A Vector of BAddress objects
    */
   public Vector doGetAddressBook()
   {
      return addressBook.getAddresses();
   }

   /**
    * A check to determine if this is the server address book manager or the client.
    *
    * @return an int representing this as the server address book or the client
    */
   public int getAddressBookLocation()
   {
      return this.addressBookLocation;
   }

   /**
    * Retrieves the address book from file
    */
	void doRestoreAddressBook()
	{
 	    try
  	    {
           Debug.doPrintln("AddressBookMgr: checking file info");
           
  	        FileInputStream fis = new FileInputStream(addressBookFileName);
	        ObjectInputStream ois = new ObjectInputStream(fis);
            
           BAddressBook tempAddressBook = (BAddressBook)ois.readObject();
            
           if (tempAddressBook != null)
              this.addressBook = tempAddressBook;
           else
              this.addressBook = new BAddressBook();

           Debug.doPrintln("AddressBookMgr: checked file info");
                
        }
        catch(StreamCorruptedException corruptedInputStream) {}
        catch(ClassNotFoundException classNotFound) {}
        catch(FileNotFoundException fileNotFound) {}
        catch(IOException ioStuffUp) {}
	}

    /**
     * Gets the name of the file to save the BAddressBook object to.
     *
     * @return The file name to save the BAddress object to.
     */
    public String doGetFileName()
    {
        return addressBookFileName;
    }

    
    /**
     * The CommandComplete class to create when dispatching a BGetNetworkStatusCE
     */
    class BGetNetworkStatusCC extends BCommandComplete
    {
        /**
         * When the users have been retrieved this method is called
         *
         * @param   event   The event that was originally fired
         * @param   status  The success status of the operation
         */
        public void doComplete( BCommandEvent event, int status )
        {
           BGetNetworkStatusCE statusEvent = (BGetNetworkStatusCE) event;

           addressBookLocation = statusEvent.getStatus();

		     if (addressBookLocation == BGetNetworkStatusCE.kClient)
             addressBookFileName = kClientBookFileName;
           else if (addressBookLocation == BGetNetworkStatusCE.kServer)
             addressBookFileName = kServerBookFileName;
           else
             Debug.doPrintln("AddressBookMgr: error whilst obtaining the network status");
        }
    }
}