

package au.edu.swin.synergy.plugin.command.event;

import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;

/**
 * This is the event which should be used whenever a general
 * dialog with the user is required. It provides a means of both
 * asking the user for a response to the query provided as well
 * as simply notifying them of something.
 * <p>
 * The handler of this event may be either the BCommandLine or the
 * BWindowManager, or anything else that registers itself as the
 * listener for BStandardOutCE's.
 *
 * @author Tristan Austin
 */
public class BStandardOutCE extends BCommandEvent
{
    /**
     * This is the message to be shown to the user
     */
    private String message = null;

    /**
     * Indicates whether the plugin requires a response from the user
     */
    private boolean requiresResponse = false;

    /**
     * Indicates that this is an error message
     */
    public static final int kError = 1;

    /**
     * Indicates that this is an information message
     */
    public static final int kInfo = 2;

    /**
     * Indicates that this is an information message
     */
    public static final int kQuestion = 3;

    /**
     * Indicates that this is a warning message
     */
    public static final int kWarning = 4;

    /**
     * This is the message type
     */
    private int messageType = kInfo;

    /**
     * Indicates the 
     */
    private int userResponseOption = kDefaultOption;

    /**
     * Indicates to use the default "ok" option for the user
     */
    public static final int kDefaultOption = 0;

    /**
     * Indicates to use the default "yes/no" option for the user
     */
    public static final int kYesNoOption = 1;

    /**
     * Indicates to use the default "yes/no/cancel" option for the user
     */
    public static final int kYesNoCancelOption = 2;

    /**
     * Indicates to use the default "ok/cancel" option for the user
     */
    public static final int kOkCancelOption = 3;

    /**
     * The users response in the affirmative
     */
    public static final int kYes = 0;

    /**
     * The users response in the negative
     */
    public static final int kNo = 1;

    /**
     * The users response in the affirmative, this is the equivalent to kYes
     * but included for ease of use reasons.
     */
    public static final int kOK = kYes;

    /**
     * The users response in the "cancelative"
     */
    public static final int kCancel = 2;

    /**
     * This is the response the user made to the message
     */
    private int usersResponse = kNo;

    /**
     * Cretaes a new BStandardOutCE to be sent to
     * the plugin responsible for such events. This constructor
     * will set the requires response flags to false as the default
     * so it may be used to simply notify the users of a message.
     *
     * @param   source  The source of the event
     * @param   message The message for the user
     */
    public BStandardOutCE(Object source, String message)
    {
        this(source, message, false, null);
    }

    /**
     * Cretaes a new BStandardOutCE to be sent to
     * the plugin responsible for such events.
     *
     * @param   source  The source of the event
     * @param   message The message for the user
     * @param   requiresResponse    True to get a response back, false otherwise
     * @param   completionObject    The class to be told when the user responds
     */
    public BStandardOutCE(Object source, String message, boolean requiresResponse, BCommandComplete completionObject)
    {
        super(source);
        this.message = message;
        this.requiresResponse = requiresResponse;
        setCompletionObject(completionObject);
    }

    /**
     * Cretaes a new BStandardOutCE to be sent to
     * the plugin responsible for such events.
     *
     * @param   source  The source of the event
     * @param   message The message for the user
     * @param   requiresResponse    True to get a response back, false otherwise
     * @param   completionObject    The class to be told when the user responds
     */
    public BStandardOutCE(Object source, String message, boolean requiresResponse, int messageType, BCommandComplete completionObject)
    {
        super(source);
        this.message = message;
        this.requiresResponse = requiresResponse;
		this.messageType = messageType;
        setCompletionObject(completionObject);
    }

    /**
     * Gets the message to be displayed to the user.
     *
     * @return The message in a String
     */
    public String getMessage()
    {
        return this.message;
    }

    /**
     * Indicates whether the user should be prompted
     * for a response to the message displayed.
     *
     * @return True if they should, false otherwise
     */
    public boolean isResponseRequired()
    {
        return this.requiresResponse;
    }

    /**
     * Gets the response the user should select
     * if they want to respond in the affirmative.
     *
     * @return The label as as a String
     */
    public int getUserResponseOption()
    {
        return this.userResponseOption;
    }

    /**
     * This will set the options the user has to response
     * to one of the predefined values provided in this class.
     *
     * @param   userResponseOption   kDefaultOption, kOkCancelOption, kYesNoCancelOption, kYesNoOption
     */
    public void setUserResponseOption(int userResponseOption)
    {
        this.userResponseOption = userResponseOption;
    }

    /**
     * Allows the handling module to set
     * the response the user selected in
     * the event so that the source of the event
     * can receive the response.
     *
     * @param   userResponse    The response from the user kYes, kNo, kCancel
     */
    public void setUserResponse(int userResponse)
    {
        this.usersResponse = userResponse;
    }

    /**
     * When the completion object is called, calling this
     * method will supply the users response to the message.
     *
     * @return One of kYes, kNo or kCancel Depending ont he users response
     */
    public int getUserResponse()
    {
        return this.usersResponse;
    }

    /**
     * Sets the type of message being displayed. Depending
     * on the value given here, the message may be represented
     * differently on screen.
     *
     * @param   messageType kInfo or kError
     */
    public void setMessageType(int messageType)
    {
        this.messageType = messageType;
    }

    /**
     * Gets the type of message being displayed. Depending
     * on the value given here, the message may be represented
     * differently on screen.
     *
     * @param   messageType kInfo or kError
     */
    public int getMessageType()
    {
        return this.messageType;
    }
}
