
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/plugin/diagram/BDrawable.java,v 1.42 1999/10/17 21:24:21 tp079568 Exp $

/**
 * Abstract superclass for all things that
 * are drawable in a diagram.
 *
 * @author Imron Alston
 *
 */

package au.edu.swin.synergy.plugin.diagram;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

import java.io.Serializable;


import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.plugin.diagram.BIconView;
import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;

import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;
import au.edu.swin.synergy.debug.Debug;

public abstract class BDrawable implements Icon, BSaveable, Serializable
{
	//flags must be on power of 2 boundaries...i.e. 1, 2, 4, 8, 16 etc..
	public static final int kEmptyFlag = 0;
	public static final int kHighlightFlag = 1;
	public static final int kSelectedFlag = 2;

	public static final int kNotHit = 0;
	public static final int kHitBounds = 1;
	public static final int kHitSizer = 2;

	protected Color sizerColour = Color.green;

	protected static final int kSizerHalfW = 3;
	protected static final int kSizerHalfH = 3;
	protected static final int kSizerNorthWest 	= kHitSizer + 1;
	protected static final int kSizerNorthEast 	= kHitSizer + 2;
	protected static final int kSizerSouthEast 	= kHitSizer + 3;
	protected static final int kSizerSouthWest 	= kHitSizer + 4;
	protected static final int kSizerStartArc 	= kHitSizer + 5;
	protected static final int kSizerEndArc 	= kHitSizer + 6;
	protected static final int kSizerCtrlPoint1	= kHitSizer + 7;
	protected static final int kSizerCtrlPoint2	= kHitSizer + 8;

	public static final int kForward = 1;
	public static final int kBackward = -1;

	protected int paintCount = 0;

	// for BSaveable
	protected static final String kKeyName = "name";
	protected static final String kDefaultName = "untitled";

	protected static final String kKeyStatus = "statusFlag";
	protected static final String kDefaultStatus = "0";


	protected BScalarPoint pos;
	protected String name = kDefaultName;

	protected BScalarRect bounds = null;
	protected BScalarRect tmpBounds = null;

	protected int statusFlag = kEmptyFlag;

	protected int lastFrame = -1;

	//don't need to save these...
	protected BScalar iconOOZ = null;

	protected BDrawable()
	{
		this.pos = new BScalarPoint( 0, 0 );
		this.bounds = new BScalarRect();
		this.tmpBounds = new BScalarRect();
		iconOOZ = new BScalar(1);
	}

	public final boolean isTransformed( int frameCounter )
	{
		return frameCounter == this.lastFrame;
	}

	/**
	 * @return true if this drawable is able to be deleted at this point in time
	 * @return false otherwise
	 *
	 */
	public String isDeletable( )
	{
		return null;
	}

	/**
	 * called before the drawable is about to be deleted
	 * so that it has time to clean itself up if need be.
	 *
	 */
	public void delete()
	{
	}



	/**
	 * Transforms the coordinates of the drawable object
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 * Subclasses should follow the following equation for
	 * tranforming relevant points etc.
	 * Subclasses should also update the lastFrame variable, to the
	 * frame counter that is passed in.
	 *
	 * xformPt.x = ((localPt.x + parentPos.x + pos.x - viewport.x) * viewport.zoom ) + viewport.halfx;
	 *
	 * @param BView viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public abstract void transform( BView viewport, BScalarPoint parentPos, int frameCounter );


	/**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public abstract void render( Graphics2D g2 );


    /**
    * Returns the associated property panel with this drawable.
    * Panel will allow user to change specific attributes of
    * this instance, if implemented.
    */
    public BPropPanel getPropertyPanel ()
    {
        return ( BPropPanel ) null;
    }

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public abstract BDrawable newInstance();

	/**
	 * Loads the BDrawable from a file
	 * @param BDataFile file - the file to load from
	 * @return boolean - true on success, false on failure.
	 */



	public boolean load(BDataFile file)
	{
		Debug.doPrintln("BDrawable load");

		name = file.getData( kKeyName, kDefaultName );
		pos = (BScalarPoint)file.autoLoad();
		bounds = (BScalarRect)file.autoLoad();		
		Debug.doPrintln("BDrawable load done");
		return true;
	}

	/**
	 * Saves the BDrawable to a file
	 * @param BDataFile file - the file to save to
	 * @return boolean - true on success, false on failure.
	 */
	public boolean save(BDataFile file)
	{
		Debug.doPrintln("BDrawable save");
		file.addData(kKeyName, name, kDefaultName);
		file.save((BSaveable) pos );
		file.save((BSaveable) bounds );		
		return true;
	}


	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public abstract void updateBounds();


	/**
	 * Tests to see if the bounds of this drawable object intersect
	 * with a given rectangle.  The bounds are assumed to be
	 * in model space.
	 * @param Rectangle bounds - the bounds to test against
	 * @return boolean - true if it does intersect, false otherwise
	 */
	public boolean intersects( BScalarRect bounds )
	{
		this.tmpBounds.setBounds( this.bounds );
		this.tmpBounds.incXY( this.pos );
//		Debug.doPrintln( "tmpBounds - " + this.tmpBounds + "\nbounds " + bounds );
		return this.tmpBounds.intersects( bounds );
	}

	public boolean isInside( BScalarRect bounds )
	{
		this.tmpBounds.setBounds( this.bounds );
		this.tmpBounds.incXY( this.pos );
		return bounds.contains( this.tmpBounds );
	}

	/**
	 * Tests to see if a point is contained within the
	 * bounds of this drawable object The bounds are assumed to be
	 * in model space.
	 * @param int x - the x coordinate of the point
	 * @param int y - the y coordinate of the point
	 * @return boolean - true if it does intersect, false otherwise
	 */
	public boolean contains( BScalarPoint pt )
	{
		this.tmpBounds.setBounds( this.bounds );
		this.tmpBounds.incXY( this.pos );
		return this.tmpBounds.contains( pt );
	}

	/**
	 * Checks to see if a point has hit the transformed values
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 * @return kHitSizer + XX - hit the XX sizer where xx is any number.. (kHitSizerXX will then be passed in the startSize dragSize methods..
 	 */
	public int checkXFormHit( Point pt )
	{
		int result = kNotHit;

		//if the object is selected, then check to see if one of the sizers was hit
		if ( this.isSelected() )
		{
			result = this.checkSizerHit( pt );
		}

		//if a sizer wasn't hit , then check to see if the normal object was hit...
		if ( result == kNotHit )
		{
			result = this.checkXFormBounds( pt );
		}

		return result;
	}

	/**
	 * Checks to see if a point has hit the transformed bounds
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 */
	public int checkXFormBounds( Point pt )
	{
		return kNotHit;
	}


	/**
	 * Checks to see if a sizer has been hit by a point
	 * Works on xformed values.
	 * Subclasses should determine appropriate places for
	 * the sizer..
	 *
	 * @param Point pt - the point to check the hit against
	 * @return kNotHit - not hit
	 * @return kHitSizer + XX hit the XX sizer where XX is any number.
	 */
	public int checkSizerHit( Point pt )
	{
		return kNotHit;
	}


	/**
	 * called when the drawable is about to be resized..
	 *
	 * @param int sizerNum - a sizerNum returned from checkSizerHit
	 */
	public void startResize( int sizerNum )
	{
	}

	/**
	 * Called as the drawable is being resized
	 *
	 */
	public void dragResize( BScalarPoint deltaXY )
	{
	}

	/**
	 * Called as the drawable is stopped being resized
	 *
	 */
	public void stopResize( )
	{
	}


	/**
	 * called when the drawable is about to be moved..
	 *
	 */
	public void startMove()
	{
	}

	/**
	 * Called as the drawable is being moved
	 *
	 */
	public void dragMove( BScalarPoint deltaXY )
	{
		this.pos.incXY( deltaXY );
	}

	/**
	 * called when the drawable has finished being moved..
	 *
	 */
	public void stopMove()
	{
		this.updateBounds();
	}

	/**
	 * called when the drawable is about to be added..
	 *
	 */
	public void startAdd()
	{
	}

	/**
	 * Called as the drawable is being added
	 *
	 */
	public void dragAdd( BScalarPoint deltaXY )
	{
	}

	/**
	 * called when the drawable has finished being added..
	 *
	 */
	public void stopAdd()
	{		
	}


	/**
	 *continue adding this drawable...
	 *
	 */
	public void continueAdd( BScalarPoint pt )
	{
	}


	


	public void paintIcon( Component c, Graphics g, int x, int y )
	{				
		int w = this.bounds.getWidth();
		int h = this.bounds.getHeight();
		this.updateBounds();

		if ( w > h )
		{
			iconOOZ.setValue( c.getWidth() - 10 );
			iconOOZ.divByW( this.bounds );			
		}
		else
		{
			iconOOZ.setValue( c.getHeight() - 10 );
			iconOOZ.divByH( this.bounds  );
		}

		if ( c instanceof BDrawableButton )
		{						
			BIconView.setHalfWH( BDrawableButton.halfWH );
			BIconView.setOOZ( iconOOZ );
			this.transform( BIconView.view, BIconView.kOrigin, paintCount++ );
			this.render( (Graphics2D)g );

			Debug.doPrintln("the bounds are " + this.bounds );
		}

	}


	public BScalarRect getBounds()
	{
		return this.bounds;
	}

	/**
	 * gets the local width of the drawable
	 */
	public int getIconWidth()
	{
		return 1;
	}

	/**
	 * gets the local height of the drawable
	 */
	public int getIconHeight()
	{
		return 1;
	}




	public void setHighlighted( boolean on )
	{
		if ( !this.isSelected() )
		{
			this.setFlag( kHighlightFlag, on );
		}

	}

	public void setSelected( boolean on )
	{
		this.setFlag( kHighlightFlag, false );
		this.setFlag( kSelectedFlag, on );

	}

	/**
	 * sets a flag on or off
	 *
	 */
	public void setFlag( int flag, boolean on )
	{
		if ( on )
		{
			this.statusFlag |= flag;
		}
		else
		{
			this.statusFlag &= ~flag;
		}

	}

//	Following 3 functions were final, until the famous hack of the BProperty.
//	They might not need be overwritten, but, just mucking around.	
	public boolean isStatusFlag( int flag )
	{
		return ( this.statusFlag & flag ) == flag;
	}

	public boolean isHighlighted()
	{
		return (this.statusFlag & kHighlightFlag) == kHighlightFlag;
	}

	public boolean isSelected()
	{
		return ( this.statusFlag & kSelectedFlag) == kSelectedFlag;
	}





	/**
	* gets the outline colour of this drawable.
	*/
	// Commented out so functions may be called but everything
	// doesn't have to be updated just yet.
//	public abstract Color getColour ();
	public Color getColour ()
	{
		return Color.red;
	}





	/**
	* gets the basic stroke of the drawable.
	*/
	// Commented out so functions may be called but everything
	// doesn't have to be updated just yet.
//	public abstract BasicStroke getBasicStroke ();
	public BasicStroke getBasicStroke ()
	{
		return new BasicStroke ();
	}





	/**
	 * Sets the position of the object.
	 */
	public void setPos( BScalarPoint pos )
	{
		this.pos.setLocation( pos );
//		this.updateBounds();
	}

	/**
	 * Sets the position of the object.
	 */
	public void setPos( Point pos )
	{
		this.pos.setLocation( pos );
//		this.updateBounds();
	}

	/**
	 *  Snaps the position of a drawable to the grid.
	 * @param int spacing - the spacing of the grid
	 */
	public void snapToGrid( int spacing )
	{
		this.pos.snap( spacing );
	}

	/**
	 * increments the position of the object.
	 */
	public void incPos( BScalarPoint deltaXY )
	{
		this.pos.incXY( deltaXY );
//		this.updateBounds();
	}

	public BScalarPoint getPos()
	{
		return this.pos;
	}


	/**
	 * toString - currently just prints the name of the object.
	 *
	 */
	public String toString()
	{
		return this.getName();
	}

	/**
	 * copies the data from the src BDrawable into this one.
	 * @param BDrawable src - the source BDrawable..
	 */
	public void copy( BDrawable src )
	{
		this.name = new String ( src.name );
		this.pos.setLocation( src.pos );
		this.bounds.setBounds( src.bounds );
	}

	/**
	 * Returns the name of this drawable object
	 * @return String - the name of the object..
	 */
	public String getName()
	{
		return this.name;
	}


	public void setName( String name )
	{
		this.name = name;
	}

	/**
	 * Called when a drawable is pasted onto a diagram.
	 * When things like model elements are pasted
	 * they need to change their package
	 */
	public void doPasted(BDiagram diagram)
	{
	}

}
