/**
 * class that is a window for a view port.
 * it handles scrolling etc.
 * @author Imron Alston
 * @copyright Team Synergy 1999
 *
 */

package au.edu.swin.synergy.plugin.diagram;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;

import au.edu.swin.synergy.plugin.diagram.BViewport;
import au.edu.swin.synergy.math.BScalarPoint;
import au.edu.swin.synergy.debug.Debug;

public class BViewWindow extends JPanel implements AdjustmentListener, MouseInputListener
{
	private static final int kSplitLen = 5;

	private static final int kLeft = 1;
	private static final int kRight = 2;
	private static final int kBoth = 3;

	private BViewport viewport = null; //the viewport the window is looking at..

	private JScrollBar horzScroll = null;//the horizontal scrollbar
	private JScrollBar vertScroll = null;//the vertical scrollbar..

	private JPanel horzSplit = null;
	private JPanel vertSplit = null;

	private BScalarPoint scrollPoint = null;
	private int oldHValue = 0;
	private int oldVValue = 0;

	private JPanel dragPanel = null;

	private BViewWindow parent = null; //the parent window...

	private BViewWindow leftChild = null; //left or top child..
	private BViewWindow rightChild = null; //right or bottom child..
	private boolean split = false;



	public BViewWindow()
	{
		this( null );
	}

	public BViewWindow( BViewport viewport )
	{
		this.addMouseListener( this );

		this.horzScroll = new JScrollBar( JScrollBar.HORIZONTAL );
		this.vertScroll = new JScrollBar( JScrollBar.VERTICAL );
		
		this.horzSplit = new JPanel();
		this.vertSplit = new JPanel();
		this.horzSplit.addMouseListener( this );
		this.horzSplit.addMouseMotionListener( this );
		this.vertSplit.addMouseListener( this );
		this.vertSplit.addMouseMotionListener( this );
		this.horzSplit.setBackground( Color.darkGray );
		this.vertSplit.setBackground( Color.darkGray );
		this.horzSplit.setBorder( BorderFactory.createRaisedBevelBorder() );
		this.vertSplit.setBorder( BorderFactory.createRaisedBevelBorder() );


		this.scrollPoint = new BScalarPoint();
		this.setViewport( viewport );

		this.setLayout( null );
		this.add( this.horzSplit, 0 );
		this.add( this.vertSplit, 1 );
		this.add( this.horzScroll );
		this.add( this.vertScroll );
	}

	static int count = 0;

	void init()
	{
		this.oldHValue = 0;
		this.oldVValue = 0;
		this.dragPanel = null;
		this.viewport = null;
		this.leftChild = null;
		this.rightChild = null;
		this.parent = null;
		this.split = false;

		this.horzScroll.setVisible( true );
		this.horzSplit.setVisible( true );
		this.vertScroll.setVisible( true );
		this.vertSplit.setVisible( true );
	}

	protected void splitHorizontal()
	{
		Debug.doPrintln("Splitting Horizontal");
		this.horzScroll.setVisible( false );
		this.vertScroll.setVisible( false );
		this.vertSplit.setVisible( false );

		this.splitWindow();
	}

	protected void splitVertical()
	{
		Debug.doPrintln("Splitting Vertical");
		this.vertScroll.setVisible( false );
		this.horzScroll.setVisible( false );
		this.horzSplit.setVisible( false );
		this.splitWindow();
	}

	protected void splitWindow()
	{
		if ( this.viewport != null )
		{
			this.remove( this.viewport );
			this.horzScroll.removeAdjustmentListener( this.viewport );
			this.vertScroll.removeAdjustmentListener( this.viewport );

			this.leftChild = BViewWindowFactory.getBViewWindow();
			this.leftChild.setParent( this );
			this.leftChild.setViewport( this.viewport );

			this.rightChild = BViewWindowFactory.getBViewWindow();
			this.rightChild.setParent( this );
			this.rightChild.setViewport( (BViewport)this.viewport.getNewView() );


			this.add( leftChild );
			this.add( rightChild );

			this.viewport = null;

			this.split = true;
		}

	}

	/**
	 * collapses the tree to a single child
	 *
	 */
	protected static void collapse( BViewWindow viewWindow )
	{
		BViewWindow curWindow = viewWindow;
		BViewWindow curParent = null;


		while ( curWindow.parent != null )
		{
			curParent = curWindow.parent;

			if ( curParent.leftChild == curWindow ) //are we the left child?
			{
				curParent.discard( curParent.rightChild );
				curParent.merge( curParent.leftChild );
			}
			else //must be the right child
			{
				curParent.discard( curParent.leftChild );
				curParent.merge( curParent.rightChild );
			}


			curWindow = curParent;
		}

		curWindow.revalidate();
	}


	/**
	 * discards a child from the split
	 *
	 */
	protected void discard( BViewWindow viewWindow )
	{
		if ( viewWindow.split )
		{

			discard( viewWindow.leftChild );
			viewWindow.remove( viewWindow.leftChild );	
			BViewWindowFactory.releaseBViewWindow( viewWindow.leftChild );
			viewWindow.leftChild = null;

			discard( viewWindow.rightChild );
			viewWindow.remove( viewWindow.rightChild );
			BViewWindowFactory.releaseBViewWindow( viewWindow.rightChild );
			viewWindow.rightChild = null;


			viewWindow.split = false;
			viewWindow.dragPanel = null;
		}
		else
		{
			viewWindow.remove( viewWindow.viewport );
			viewWindow.horzScroll.removeAdjustmentListener( viewWindow.viewport );
			viewWindow.vertScroll.removeAdjustmentListener( viewWindow.viewport );
		}



	}

	
	/**
	 * merges a child from the split
	 */
	protected void merge( BViewWindow viewWindow )
	{
		BViewWindow tmpWindow = null;

		//remove the children from the component..
		this.remove( this.leftChild );
		this.remove( this.rightChild );
		this.leftChild.horzScroll.removeAdjustmentListener( this.leftChild );
		this.leftChild.vertScroll.removeAdjustmentListener( this.leftChild );
		this.rightChild.horzScroll.removeAdjustmentListener( this.rightChild );
		this.rightChild.vertScroll.removeAdjustmentListener( this.rightChild );


		if ( viewWindow.split ) //if the child window was split, then update the children..
		{
			//was viewwindow a horizontal split?
			if ( viewWindow.dragPanel == viewWindow.horzSplit )
			{
				this.horzSplit.setBounds( 0, viewWindow.horzSplit.getY(), this.getWidth(), kSplitLen );
				this.horzSplit.setVisible( true );
				this.vertSplit.setVisible( false );
				this.dragPanel = this.horzSplit;
			}
			else //must be a vertical split..
			{
				this.vertSplit.setBounds( viewWindow.vertSplit.getX(), 0, kSplitLen, this.getHeight() );
				this.horzSplit.setVisible( false );
				this.vertSplit.setVisible( true );
				this.dragPanel = this.vertSplit;
			}

			viewWindow.leftChild.setParent( this.parent );
			viewWindow.rightChild.setParent( this.parent );

			//are we merging left or right
			if ( viewWindow == this.leftChild )
			{
				BViewWindowFactory.releaseBViewWindow( this.rightChild );
                this.rightChild = viewWindow.rightChild;

				tmpWindow = this.leftChild;
				this.leftChild = viewWindow.leftChild;
				BViewWindowFactory.releaseBViewWindow( tmpWindow );
			}
			else //must be merging right...
			{
				BViewWindowFactory.releaseBViewWindow( this.leftChild );
				this.leftChild = viewWindow.leftChild;

				tmpWindow = this.rightChild;
				this.rightChild = viewWindow.rightChild;
				BViewWindowFactory.releaseBViewWindow( tmpWindow );
			}

			this.add( this.leftChild );
			this.add( this.rightChild );

		}
		else //viewwindow isn't split, so just get rid of this' children..
		{
			
			if ( viewWindow == this.leftChild ) // left child
			{
				this.setViewport( this.leftChild.viewport );	
			}
			else //must be the right child
			{
				this.setViewport( this.rightChild.viewport );
			}
			this.leftChild.viewport = null;
			this.rightChild.viewport = null;

			BViewWindowFactory.releaseBViewWindow( this.leftChild );
			this.leftChild = null;

			BViewWindowFactory.releaseBViewWindow( this.rightChild );
			this.rightChild = null;

			this.horzScroll.setVisible( true );
			this.horzSplit.setVisible( true );
			this.vertScroll.setVisible( true );
			this.vertSplit.setVisible( true );

			this.split = false;
			this.dragPanel = null;
		}
	}



	/**
	 * sets the viewport of this viewportwindow..
	 */
	public final void setViewport( BViewport viewport )
	{
		this.viewport = viewport;
		if ( viewport != null )
		{
			this.add( viewport );
			this.horzScroll.addAdjustmentListener( viewport );
			this.vertScroll.addAdjustmentListener( viewport );

		}
	}

	public final BViewport getViewport ()
	{
		return this.viewport;
	}

	public final void setParent( BViewWindow parent )
	{
		this.parent = parent;
	}

	public final void updateViewports()
	{
		if ( parent != null )
		{			
			this.parent.updateParent( this );			
		}

		if ( this.split )
		{		
			this.leftChild.updateChildren();		
			this.rightChild.updateChildren();			
		}
	}


	public final void updateParent( BViewWindow source )
	{
		if ( parent != null )
		{	
			this.parent.updateParent( this );
		}

		if ( split )
		{
			if ( source != this.leftChild )
			{
				this.leftChild.updateChildren();
			}

			if ( source != this.rightChild )
			{
				this.rightChild.updateChildren();
			}
		}
	}

	public final void updateChildren()
	{
		if ( !this.split )
		{
			this.viewport.specialRepaint();

		}
		else
		{
			this.leftChild.updateChildren();
			this.rightChild.updateChildren();	
		}
	}


	/**
	 * does the layout of this window...
	 */
	public void doLayout()
	{
		if ( this.split )
		{
			this.doSplitLayout();
		}
		else
		{
			this.doNormalLayout();
		}
	}


	/**
	 * Draws the BViewWindow when it is split...
	 *
	 */
	protected void doSplitLayout()
	{
		int x, y, width, height;

		if ( this.dragPanel == this.horzSplit )
		{
			x = 0;
			y = 0;
			width = this.getWidth();
			height = this.dragPanel.getY();
			this.leftChild.setBounds( x, y, width, height );

			x = 0;
			y = this.dragPanel.getY() + this.dragPanel.getHeight();
			width = this.getWidth();
			height = this.getHeight() - y;
			this.rightChild.setBounds( x, y, width, height );

			this.dragPanel.setBounds( 0, this.horzSplit.getY(), this.getWidth(), kSplitLen );


		}
		else if ( this.dragPanel == this.vertSplit )
		{
			x = 0;
			y = 0;
			width = this.dragPanel.getX();
			height = this.getHeight();
			this.leftChild.setBounds( x, y, width, height );

			x = this.dragPanel.getX() + this.dragPanel.getWidth();
			y = 0;
			width = this.getWidth() - x;
			height = this.getHeight();
			this.rightChild.setBounds( x, y, width, height );

			this.dragPanel.setBounds( this.vertSplit.getX(), 0, kSplitLen, this.getHeight() );
		}
	}

	/**
	 * draws the BViewWindow without any splits in it..
	 *
	 */
	protected void doNormalLayout()
	{
		int hScrollY, hScrollW;
		int vScrollX, vScrollH;

		//calculate sizes for the components..
		Dimension d1 = this.horzScroll.getPreferredSize();
		Dimension d2 = this.vertScroll.getPreferredSize();
		hScrollY = this.getHeight() - d1.height;
		vScrollX = this.getWidth() - d2.width;
		hScrollW = vScrollX;
		vScrollH = hScrollY;

		//if the splitter isn't being dragged, then set the bounds to the default position
		if ( this.vertSplit != this.dragPanel)
		{
			this.vertSplit.setBounds( 0, hScrollY, kSplitLen, d1.height );
		}
		this.horzScroll.setBounds( kSplitLen, hScrollY, hScrollW - kSplitLen, d1.height );

		//if the splitter isn't being dragged, then set the bounds to the default position
		if ( this.horzSplit != this.dragPanel )
		{
			this.horzSplit.setBounds( vScrollX, 0, d2.width, kSplitLen );
		}
		this.vertScroll.setBounds( vScrollX, kSplitLen, d2.width, vScrollH - kSplitLen );

		//set the bounds of the viewport if we have one
		if ( this.viewport != null )
		{
			this.viewport.setBounds( 0, 0, hScrollW, vScrollH );
		}
	}

	/**
	 * What to do when the scroll bar is clicked...
	 */
	public void adjustmentValueChanged( AdjustmentEvent e )
	{
		Adjustable source = e.getAdjustable();
		int value = e.getValue();
		int dv;

		//don't do anything if there isn't a viewpot..
		if ( this.viewport == null ) return;

		if ( source.getOrientation() == Adjustable.VERTICAL )
		{
			dv = this.oldVValue - value; //do this backwards to get -ve value :)
			this.scrollPoint.setLocation( 0, dv );
			this.oldVValue = value;
		}
		else if (source.getOrientation() == Adjustable.HORIZONTAL )
		{
			dv = this.oldHValue - value; //do this backwards to get -ve value :)
			this.scrollPoint.setLocation( dv, 0 );
			this.oldHValue = value;

		}

		this.viewport.scroll( this.scrollPoint );
	}


    public void mouseClicked( MouseEvent e )
    {
    }
    /**
     * Please complete the missing tags for mouseEntered
     * @param
     * @return
     * @pre
     * @post
     */

    public void mouseEntered( MouseEvent e )
    {
    }
    /**
     * Please complete the missing tags for mouseExited
     * @param
     * @return
     * @pre
     * @post
     */

    public void mouseExited( MouseEvent e )
    {
    }


    /**
     * Please complete the missing tags for mousePressed
     * @param
     * @return
     * @pre
     * @post
     */
    public void mousePressed( MouseEvent e )
    {
		Object source = e.getSource();

		if ( source == this.horzSplit )
		{
			this.dragPanel = this.horzSplit;
			this.dragPanel.setSize( this.getWidth(), this.dragPanel.getHeight() );
		}
		else if ( source == this.vertSplit )
		{
			this.dragPanel = this.vertSplit;
			this.dragPanel.setSize( this.dragPanel.getWidth(), this.getHeight() );
		}

    }


    /**
     * Please complete the missing tags for mouseReleased
     * @param
     * @return
     * @pre
     * @post
     */

    public void mouseReleased( MouseEvent e )
    {
		Object source = e.getSource();
		int pos, max, min;

		if ( source == this.horzSplit )
		{
			min = 0;
			pos = this.horzSplit.getY();
			max = this.getHeight();

			if ( !this.split && pos > min && pos < max ) //if not split, then split it...
			{
				this.splitHorizontal();
			}
			else if ( pos < min && this.split)// perhaps we have to unsplit it??
			{
				this.discard( this.leftChild );
				this.merge( this.rightChild );

			}
			else if ( pos > max && this.split )
			{
				this.discard( this.rightChild );
				this.merge( this.leftChild );
			}
			this.revalidate();
		}
		else if ( source == this.vertSplit )
		{
			min = 0;
			pos = this.vertSplit.getX();
			max = this.getWidth();

			if ( !this.split && pos > min && pos < max )
			{
				this.splitVertical();
			}
			else if ( pos < min && this.split )// perhaps we have to unsplit it??
			{
				this.discard( this.leftChild );
				this.merge( this.rightChild );
			}
			else if ( pos > max  && this.split)
			{
				this.discard( this.rightChild );
				this.merge( this.leftChild );
			}
			this.revalidate();

		}
		else if ( source == this ) // need to collapse
		{

			if ( e.getClickCount() >= 2 )
			{
				//the this pointer could be on the free list after call..
				collapse( this );
			}
		}


	}
    /**
     * Please complete the missing tags for mouseDragged
     * @param
     * @return
     * @pre
     * @post
     */

    public void mouseDragged( MouseEvent e )
    {
		Object source = e.getSource();
		Point pt = e.getPoint();

		if ( source == this.horzSplit )
		{
			this.dragPanel.setLocation( 0, this.dragPanel.getY() + pt.y );
		}
		else if ( source == this.vertSplit )
		{
			this.dragPanel.setLocation( this.dragPanel.getX() + pt.x, 0 );
		}
    }
    /**
     * Please complete the missing tags for mouseMoved
     * @param
     * @return
     * @pre
     * @post
     */

    public void mouseMoved( MouseEvent e )
    {

    }

}