

package au.edu.swin.synergy.plugin.dialect;

import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.event.BBootCE;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.plugin.prefs.event.BAddPrefsCE;
import au.edu.swin.synergy.plugin.prefs.event.BUpdatePrefsCE;

import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;


import java.util.Locale;
import java.util.Hashtable;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

import java.text.MessageFormat;
import java.util.StringTokenizer;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

/**
 * The dialect class provides a common location
 * for loading language resources from specific
 * or global resources bundles.
 * <p>
 * This primarily consists of static methods which
 * can be accessed without the need for command events
 * however it will also add a language preferences panel
 * to the Preferences plugin which will allow the user
 * to change the current language to another.
 * <p>
 * When this happens, an dialect change event will be
 * broadcast to all plugins on the client and it is up to
 * them to update their users interfaces by requesting
 * the required language resources from this plugin.
 *
 * @author Tristan Austin
 */
public class BDialect extends BPlugin
{
    /**
     * This contains all the resources bundles that have been loaded
     */
    static Hashtable resourceBundles = new Hashtable();

    /**
     * This is the default name the property files should have
     */
    public static final String kScreenTextFileName = "screentext";

    /**
     * This is the file name extension for language resource files
     */
    protected static final String kResourceExtension = ".properties";

    /**
     * This is the default name the global property files will have
     */
    public static final String kGlobalTextFileName = "globaltext";

    /**
     * The path of the preferences file
     */
    public static final String kPreferencesFile = "./au/edu/swin/synergy/plugin/dialect/dialect.cfg";

    /**
     * The key in the preferences file for the locale
     */
    public static final String kSavedLocale = "locale";

    /**
     * This the preferences panel as set on the preferences plugin
     */
    BDialectPreferences preferences = new BDialectPreferences(this);

    /**
     * This will load up the current locale if one has already been saved
     */
    static
    {
        //Load up the last used Locale
        doLoad();
    }

    /**
     * This version of getString will search the global text
     * language resource file for the text mapping to the given key.
     * <p>
     * In addition, it will take the arguments array and format the text
     * before returning it initially.
     * <P>
     * Because the global properties file is defined by the BDialect plugin
     * you don't need to specify any file or location, it already knows.
     *
     * @param   key The key to be mapped to a value in the resource file
     * @return The string mapped to the give key in the global resource file
     */
    public String getString(String key, Object args[])
    {
    	String text = getString(key);
	return formatText(text, args);
    }

    /**
     * This version of getString will search the global text
     * language resource file for the text mapping to the given key.
     * <p>
     * Because the global properties file is defined by the BDialect plugin
     * you don't need to specify any file or location, it already knows.
     *
     * @param   key The key to be mapped to a value in the resource file
     * @return The string mapped to the give key in the global resource file
     */
    public static String getString(String key)
    {
        return getString(key, Locale.getDefault());
    }

    /**
     * This version of getString will search the global text
     * language resource file for the text mapping to the given key.
     * <p>
     * Because the global properties file is defined by the BDialect plugin
     * you don't need to specify any file or location, it already knows.
     *
     * @param   key The key to be mapped to a value in the resource file
     * @param   locale  The locale to get the text for
     * @return The string mapped to the give key in the global resource file
     */
    public static String getString(String key, Locale locale)
    {
        //Use the global text file
        String text = null;
        String baseName;
        ResourceBundle resourceBundle;

        //This is the key to the hashtable
        if (resourceBundles.containsKey(kGlobalTextFileName + locale.toString()))
        {
            resourceBundle = (ResourceBundle)resourceBundles.get(kGlobalTextFileName + locale.toString());
        }
        else
        {
            //Load the resource bundle
            baseName = "au.edu.swin.synergy.plugin.dialect." + kGlobalTextFileName;
            resourceBundle = ResourceBundle.getBundle(baseName, locale);

            //Need to make sure it was found
            if (resourceBundle != null)
            {
                resourceBundles.put(kGlobalTextFileName + locale.toString(), resourceBundle);
            }
        }

        //If we have a resource bundle, get the value
        if (resourceBundle != null)
        {
            try
            {
                //Get the tag
                text = (String)resourceBundle.getString(key);
            }
            catch(MissingResourceException missingResource)
            {
                //We want to handle this gracefully
                text = "UV: " + key;
            }
        }

        //Return the given value
        return text;
    }

    /**
     * This will find the screentext<>.properties file
     * in the same directory as the class is defined
     * of the object passed in.
     * <p>
     * For example, if the plugin au.edu.swin.synergy.plugin.mod13.BWindowManager
     * wants to load a resource, it will pass in an instance of the window manager,
     * or anything else declared in the same package.
     * <p>
     * The package is taken from it, converted to a path and the basename is
     * appended to the end of the package. It will therefore try
     * and locate it in the directory: ./au/edu/swin/synergy/plugin/mod13/screentext.properties
     * or a more specific version for specific locales.
     *
     * @param   object  The object who's package indicates the location of the resource file
     * @param   key The key to be mapped to a value.
     * @return The string value mapped to the given key for the current locale
     */
    public static String getString(Object object, String key, Object args[])
    {
    	String text = getString(object, key);
    	return formatText(text, args);
    }


    /**
     * This will find the screentext<>.properties file
     * in the same directory as the class is defined
     * of the object passed in.
     * <p>
     * For example, if the plugin au.edu.swin.synergy.plugin.mod13.BWindowManager
     * wants to load a resource, it will pass in an instance of the window manager,
     * or anything else declared in the same package.
     * <p>
     * The package is taken from it, converted to a path and the basename is
     * appended to the end of the package. It will therefore try
     * and locate it in the directory: ./au/edu/swin/synergy/plugin/mod13/screentext.properties
     * or a more specific version for specific locales.
     *
     * @param   object  The object who's package indicates the location of the resource file
     * @param   key The key to be mapped to a value.
     * @return The string value mapped to the given key for the current locale
     */
    public static String getString(Object object, String key)
    {
        return getString(object, key, Locale.getDefault());
    }

    /**
     * This will find the screentext<>.properties file
     * in the same directory as the class is defined
     * of the object passed in.
     * <p>
     * For example, if the plugin au.edu.swin.synergy.plugin.mod13.BWindowManager
     * wants to load a resource, it will pass in an instance of the window manager,
     * or anything else declared in the same package.
     * <p>
     * The package is taken from it, converted to a path and the basename is
     * appended to the end of the package. It will therefore try
     * and locate it in the directory: ./au/edu/swin/synergy/plugin/mod13/screentext.properties
     * or a more specific version for specific locales.
     *
     * @param   object  The object who's package indicates the location of the resource file
     * @param   key The key to be mapped to a value.
     * @param   locale  The locale to get the text for
     * @return The string value mapped to the given key for the current locale
     */
    public static String getString(Object object, String key, Locale locale)
    {
        String text = null;
        ClassLoader classLoader;
        Class theClass;
        ResourceBundle resourceBundle;
        String baseName;
        int index;

        try
        {
            //This is the key to the hashtable
            if (resourceBundles.containsKey(object + locale.toString()))
            {
                resourceBundle = (ResourceBundle)resourceBundles.get(object + locale.toString());
            }
            else
            {
                //Load the resource bundle
                theClass = object.getClass();
                baseName = theClass.getName();
                index = baseName.lastIndexOf('.');

                if (index >= 0)
                {
                    baseName = baseName.substring(0, index +1);
                }

                baseName += kScreenTextFileName;
                resourceBundle = ResourceBundle.getBundle(baseName, locale);
                if (resourceBundle != null)
                {
                    resourceBundles.put(object + locale.toString(), resourceBundle);
                }
            }

            //If we have a resource bundle, get the value
            if (resourceBundle != null)
            {
                //Get the tag
                text = (String)resourceBundle.getString(key);
            }
        }
        catch(MissingResourceException missingResource)
        {
            //Look in the global resource first
            text = getString(key, locale);
        }

        //This will search for the local file, then the global file
        return text;
    }

    /**
     * This will be called when the user changes the
     * current dialect. It will make sure there is a change
     * and update the contents of the resourceBundles hashtable
     * as well as notify all the other plugins.
     */
    protected void doDialectChange()
    {
        //Check to see if they're different
        Locale newLocale = this.preferences.getNewLocale();

        if (!newLocale.equals(Locale.getDefault()))
        {
            //Clear the hashtable
            resourceBundles.clear();

            //Set the new locale
            Locale.setDefault(newLocale);

            //Notify all plugins of change
            BDialectChangeCE changeEvent = new BDialectChangeCE(this);
            this.dispatchEvent(changeEvent);
        }
    }

    /**
     * When dealing with compound messages, you need to specify the location
     * and type of the arguments in the compound message in the following form:
     * <P>
     * eg.<BR>
     * At {2,time,short} on {2,date,long}, we detected {1,number,integer} spaceships on the planet {0}.
     * <P>
     * Where {2, time, short} indicates the third argument in the argument array, it is a time and short is the short date format<BR>
     * {2, date, long} indicates the third argument again, date says it's in date format, and long says long date format<BR>
     * {1, number, integer} Indicates second argument in array, it's a number and should be an integer (as opposed to float)<BR>
     * {0} The first String in the argument array
     * <P>
     * @param   formatMe    The compound string to format
     * @param   arguments   The arguments for the compound statement
     */
    public static String formatText(String formatMe, Object[] arguments)
    {
        MessageFormat formatter = new MessageFormat(formatMe);

        //Do the formatting and return the result
        return formatter.format(arguments);
    }

    /**
     * When events are dispatched by the kernel, they are received
     * here. Currently don't handle anything.
     */
    public void onBCommandEvent( Object source, BCommandEvent event )
    {
        if (event instanceof BBootCE)
        {
            //Register the preferences panel
            BAddPrefsCE addPrefs = new BAddPrefsCE(this, this.preferences, new BString ( this, "dialect" ), new BString ( this, "tooltip" ) );
            this.dispatchEvent(addPrefs);
        }
        else if (event instanceof BDialectChangeCE)
        {
            preferences.doDialect();
        }
        else if (event instanceof BUpdatePrefsCE)
        {
            if(((BUpdatePrefsCE)(event)).isOk())
            {
                this.doDialectChange();

                //Need to save the current value
                this.doSave();
            }
        }
    }

    /**
     * This will write the current locale to a file so that
     * we can start up with the last locale used next time.
     */
    protected void doSave()
    {
        try
        {
            OutputStream output = new FileOutputStream(kPreferencesFile);
            Properties preferences = new Properties();

            preferences.setProperty(kSavedLocale, Locale.getDefault().toString());
            preferences.store(output, "Last used Locale");
        }
        catch(IOException ioException)
        {
            Debug.doPrintln("Locale save error: ", ioException);
        }
    }

    /**
     * This will load up the last used locale so that
     * we can start up with the last locale used next time.
     */
    protected static void doLoad()
    {
        try
        {
            InputStream input = new FileInputStream(kPreferencesFile);
            Properties preferences = new Properties();

            if (input != null)
            {
                preferences.load(input);
                String localeBits[] = new String[]{"", "", ""};
                int i = 0;

                String currentLocale = (String)preferences.get(kSavedLocale);
                Debug.doPrintln("Locale read is: " + currentLocale);
                StringTokenizer tokenizer = new StringTokenizer(currentLocale, "_");

                //Get the contents
                while(tokenizer.hasMoreTokens())
                {
                    localeBits[i++] = tokenizer.nextToken();
                }

                Locale newLocale = new Locale(localeBits[0], localeBits[1], localeBits[2]);
                Locale.setDefault(newLocale);
            }
        }
        catch(FileNotFoundException fnfException)
        {/*Ignore*/}
        catch(IOException ioException)
        {
            Debug.doPrintln("Locale load error: ", ioException);
        }
    }

    /**
     * Called by the kernel when the plugin is first loaded
     */
    public void setup(BStringList params)
    {
    }

    /**
     * Return a list of plugin dependancies
     *
     * @return a list of plugins such as "au.edu.swin.synergy.plugin.mod13.BWindowManager"
	 * or null if there are no dependancies
     */

    public BStringList getDependancies()
	{
		BStringList result = new BStringList();
		result.add("au.edu.swin.synergy.plugin.prefs.BPrefs");
		return result;
	}
}
