package au.edu.swin.synergy.plugin.editmanager;

import au.edu.swin.synergy.plugin.editmanager.event.BRegisterUndoCE;

import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BPlugin;
import au.edu.swin.synergy.kernel.event.BBootCE;

import au.edu.swin.synergy.plugin.dialect.BDialect;
import au.edu.swin.synergy.plugin.dialect.BString;
import au.edu.swin.synergy.plugin.dialect.event.BDialectChangeCE;

import au.edu.swin.synergy.plugin.mod13.event.BAddMenuItemCE;
import au.edu.swin.synergy.plugin.mod13.event.BAddToolBarCE;
import au.edu.swin.synergy.plugin.mod13.event.BRemoveToolBarCE;
import au.edu.swin.synergy.plugin.mod13.BWindowManager;
import au.edu.swin.synergy.plugin.mod13.bwt.BSwingUtilities;

import au.edu.swin.synergy.lib.BStringList;

import au.edu.swin.synergy.debug.Debug;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import java.util.Date;
import java.net.URL;
import java.util.Hashtable;
import javax.swing.undo.UndoableEditSupport;
import javax.swing.event.UndoableEditListener;
import javax.swing.event.UndoableEditEvent;

/**
 * The edit manager is responsible for dispatching cut, copy, paste and undo
 * events to the component with the current focus. It is the responsibility of
 * the current component to deal with these events as they see fit. This will
 * provide backward compatability with the existing swing classes which
 * handle control-c, x, p and z already.
 *
 * @author Tristan Austin
 */
public class BEditManager extends BPlugin
{
	/**
	 * Cut menu item
	 */
	JMenuItem cutMenuItem = new JMenuItem();

	/**
	 * Copy menu item
	 */
	JMenuItem copyMenuItem = new JMenuItem();

	/**
	 * Paste menu item
	 */
	JMenuItem pasteMenuItem = new JMenuItem();

	/**
	 * undo menu item
	 */
	JMenuItem undoMenuItem = new JMenuItem();

	/**
	 * redo menu item
	 */
	JMenuItem redoMenuItem = new JMenuItem();

    /**
	 * Delete menu item
	 */
	JMenuItem deleteMenuItem = new JMenuItem();

    /**
	 * Select All menu item
	 */
	JMenuItem selectAllMenuItem = new JMenuItem();
	
	/**
	 * redo menu item
	 */
	BString localEvents[] = new BString[7];
	
	/**
	 * The action listener for the menus
	 */
	BActionListener actionListener = new BActionListener();

	/**
	 * These are the events that we have dispatched, if we get them back we consume them
	 */
	Hashtable undoManagers = new Hashtable();

	/**
	 * The current undo manager
	 */
	BUndoManager currentUndoManager;

    /**
     * The cut button
     */
    JButton cutButton;

    /**
     * The copy button
     */
    JButton copyButton;

    /**
     * The paste button
     */
    JButton pasteButton;

    /**
     * The undo button
     */
    JButton undoButton;

    /**
     * The redo button
     */
    JButton redoButton;

	/**
	 * This is where all events are received through.
	 */
	public void onBCommandEvent(Object source, BCommandEvent event)
	{
		if (event instanceof BDialectChangeCE)
		{
			this.doDialect();
		}
		else if (event instanceof BRegisterUndoCE)
		{
			this.doUndoRegistration((BRegisterUndoCE)event);
		}
    }

	/**
	 * This will create an undo listener for the source of the event
	 * and return the listener in the event itself. It is up to the source
	 * to inform the edit manager when it should be used.
	 *
	 * @oaram
	 */
	protected synchronized void doUndoRegistration(BRegisterUndoCE registerCE)
	{
		Object source = registerCE.getSource();
        Debug.doPrintln("Received undo registration");
		//Update the current undo manager and
        UndoableEditSupport editSupport = new UndoableEditSupport(source);
        currentUndoManager = new BUndoManager(this);
        editSupport.addUndoableEditListener(currentUndoManager);

		this.undoManagers.put(source, currentUndoManager);
		registerCE.setUndoSupport(editSupport);
        registerCE.setUndoListener(currentUndoManager);
		registerCE.doComplete(BCommandComplete.kSuccess);
	}

    /**
     * This will remove the undo manager mapped to the given source
     *
     * @param   source  The source of the undo manager
     */
    protected void unregisterUndoManager(Object source)
    {
        Object oldUndoManager = this.undoManagers.remove(source);
        
        if (this.currentUndoManager == oldUndoManager)
        {
            this.currentUndoManager = null;
            this.setUndoItems();
        }
    }

	/**
	 * When called, this will update the undo and redo menu items
	 * appropriate if the undo manager in which the edit occured
	 * is also the current edit manager.
	 *
	 * @param	undoManager	The undo manager that was updated
	 */
    public void editOccured(BUndoManager undoManager)
	{
		//If an undo event happened in an editor other than the currentone
        //Switch to that one and update the menu items to reflect it
		if (this.currentUndoManager != undoManager)
		{
            this.currentUndoManager = undoManager;
		}

        this.setUndoItems();
	}

	/**
	 * This will change the current undo manager to the one specified
	 *
	 * @param	undoManager	The manager to make current
	 */
	protected synchronized void setUndoManager(BUndoManager undoManager)
	{
		this.currentUndoManager = undoManager;
		this.setUndoItems();
	}

    /**
     * This will remove the current undo manager if it is this one
     */
    public void relinquishUndoManager(BUndoManager undoManager)
    {
        if (this.currentUndoManager == undoManager)
        {
            this.currentUndoManager = null;
            this.setUndoItems();
        }
    }

	/**
	 * This will set the appropriate text and enabled states on the undo
	 * and redo menu items as per the current undo manager.
	 */
	protected void setUndoItems()
	{
        boolean canUndo = false;
        boolean canRedo = false;
        if (this.currentUndoManager != null)
        {
            this.undoMenuItem.setText(this.currentUndoManager.getUndoPresentationName());
            this.redoMenuItem.setText(this.currentUndoManager.getRedoPresentationName());

            canUndo = this.currentUndoManager.canUndo();
            canRedo = this.currentUndoManager.canRedo();
        }

        this.undoMenuItem.setEnabled(canUndo);
        this.undoButton.setEnabled(canUndo);
        this.redoMenuItem.setEnabled(canRedo);
        this.redoButton.setEnabled(canRedo);
	}

	/**
	 * Sets the enabled state of the paste menu
	 * items to the state specified.
	 *
	 * @param	enabled		True to enable, false to disable
	 */
	protected void setPasteEnabled(boolean enabled)
	{
		this.pasteMenuItem.setEnabled(enabled);
	}
	
	/**
	 * Sets the enabled state of the cut and copy menu
	 * items to the state specified.
	 *
	 * @param	enabled		True to enable, false to disable
	 */
	protected void setCutCopyEnabled(boolean enabled)
	{
		this.cutMenuItem.setEnabled(enabled);
		this.copyMenuItem.setEnabled(enabled);
	}

	/**
	 * This is called when the plugin is first loaded. It is responsible for registering
	 * the menu items and setting everything up.
	 *
	 * @param	params	The paramaters supplied by the configuration file
	 */
	public void setup(BStringList params)
	{
		BAddMenuItemCE addMenuItems = new BAddMenuItemCE(this, new BString("edit"));

		localEvents[0] = new BString(this, "cut");
 		localEvents[1] = new BString(this, "copy");
 		localEvents[2] = new BString(this, "paste");
 		localEvents[3] = new BString(this, "undo");
 		localEvents[4] = new BString(this, "redo");
        localEvents[5] = new BString(this, "delete");
        localEvents[6] = new BString(this, "selectAll");


        addMenuItems.addMenuItem(undoMenuItem, localEvents[3]);
		addMenuItems.addMenuItem(redoMenuItem, localEvents[4]);
        addMenuItems.addSeparator();
		addMenuItems.addMenuItem(cutMenuItem, localEvents[0]);
		addMenuItems.addMenuItem(copyMenuItem, localEvents[1]);
		addMenuItems.addMenuItem(pasteMenuItem, localEvents[2]);
        addMenuItems.addSeparator();
        addMenuItems.addMenuItem(deleteMenuItem, localEvents[5]);
		addMenuItems.addMenuItem(selectAllMenuItem, localEvents[6]);


		cutMenuItem.addActionListener(this.actionListener);
		copyMenuItem.addActionListener(this.actionListener);
		pasteMenuItem.addActionListener(this.actionListener);
		undoMenuItem.addActionListener(this.actionListener);
		redoMenuItem.addActionListener(this.actionListener);
        deleteMenuItem.addActionListener(this.actionListener);
        selectAllMenuItem.addActionListener(this.actionListener);

		undoMenuItem.setEnabled(false);
		redoMenuItem.setEnabled(false);

		this.dispatchEvent(addMenuItems);

        cutButton = new JButton(BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/editmanager/images/cut.gif"));
        cutButton.addActionListener(this.actionListener);
        cutButton.setRequestFocusEnabled(false);

        copyButton = new JButton(BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/editmanager/images/copy.gif"));
        copyButton.addActionListener(this.actionListener);
        copyButton.setRequestFocusEnabled(false);

        pasteButton = new JButton(BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/editmanager/images/paste.gif"));
        pasteButton.addActionListener(this.actionListener);
        pasteButton.setRequestFocusEnabled(false);

        undoButton = new JButton(BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/editmanager/images/undo.gif"));
        undoButton.addActionListener(this.actionListener);
        undoButton.setRequestFocusEnabled(false);

        redoButton = new JButton(BSwingUtilities.getImage(this, "/au/edu/swin/synergy/plugin/editmanager/images/redo.gif"));
        redoButton.addActionListener(this.actionListener);
        redoButton.setRequestFocusEnabled(false);

        BAddToolBarCE addToolBar = new BAddToolBarCE(this, 100);
        addToolBar.addButton(cutButton);
        addToolBar.addButton(copyButton);
        addToolBar.addButton(pasteButton);
        addToolBar.addSeparator();
        addToolBar.addButton(undoButton);
        addToolBar.addButton(redoButton);

        //Disable the undo buttons
        undoButton.setEnabled(false);
        redoButton.setEnabled(false);

        //Set the text for the screen.
        this.doDialect();

        //Dispatches the event to the window manager
        this.dispatchEvent(addToolBar);
	}

    /**
     * Return a list of plugin dependancies
	 *
     * @return a list of plugins such as "au.edu.swin.synergy.plugin.mod13.BWindowManager"
	 * or null if there are no dependancies
     */

    public BStringList getDependancies()
	{
		BStringList result = new BStringList();
		result.add("au.edu.swin.synergy.plugin.mod13.BWindowManager");
		return result;
	}

	/**
	 * This will set the text of the menus to the correct values for the current Locale
	 */
	protected void doDialect()
	{
		char cut = BDialect.getString(this, "cutmn").charAt(0);
		char copy = BDialect.getString(this, "copymn").charAt(0);
		char paste = BDialect.getString(this, "pastemn").charAt(0);
		char undo = BDialect.getString(this, "undomn").charAt(0);
		char redo = BDialect.getString(this, "redomn").charAt(0);
        char delete = BDialect.getString(this, "deletemn").charAt(0);
        char selectAll = BDialect.getString(this, "selectallmn").charAt(0);

		KeyStroke cutKS = KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK);
		KeyStroke copyKS = KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK);
		KeyStroke pasteKS = KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK);
		KeyStroke undoKS = KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK);
		KeyStroke redoKS = KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK);
        KeyStroke deleteKS = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);
        KeyStroke selectAllKS = KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK);


		//Set the mnemonic for the menu item
        BSwingUtilities.setMnemonic(cutMenuItem, cut);
        BSwingUtilities.setMnemonic(copyMenuItem, copy);
        BSwingUtilities.setMnemonic(pasteMenuItem, paste);
        BSwingUtilities.setMnemonic(undoMenuItem, undo);
        BSwingUtilities.setMnemonic(redoMenuItem, redo);
        BSwingUtilities.setMnemonic(deleteMenuItem, delete);
        BSwingUtilities.setMnemonic(selectAllMenuItem, selectAll);

        //Set the tool tips for the buttons
        BSwingUtilities.setToolTip(cutButton, BDialect.getString(this, "cuttooltip"));
        BSwingUtilities.setToolTip(copyButton, BDialect.getString(this, "copytooltip"));
        BSwingUtilities.setToolTip(pasteButton, BDialect.getString(this, "pastetooltip"));
        BSwingUtilities.setToolTip(undoButton, BDialect.getString(this, "undotooltip"));
        BSwingUtilities.setToolTip(redoButton, BDialect.getString(this, "redotooltip"));

		//Set the accelerator (should always be the same)
        BSwingUtilities.setMenuAccererator(cutMenuItem, cutKS);
        BSwingUtilities.setMenuAccererator(copyMenuItem, copyKS);
        BSwingUtilities.setMenuAccererator(pasteMenuItem, pasteKS);
        BSwingUtilities.setMenuAccererator(undoMenuItem, undoKS);
        BSwingUtilities.setMenuAccererator(redoMenuItem, redoKS);
        BSwingUtilities.setMenuAccererator(deleteMenuItem, deleteKS);
	}

	/**
	 * This will dispatch the appropriate key event to the
	 * currently focussed component.
	 *
	 * @param	keyCode	The key code to send
	 */
	protected void dispatchEvent(JComponent focusedComponent, int keyCode, int modifierMask)
	{
		KeyEvent keyEvent;
		ActionListener actionListener;

		//Don't send it if nothing has the focus
		if (focusedComponent != null)
		{
			KeyStroke keyStroke = KeyStroke.getKeyStroke(keyCode, modifierMask);
			actionListener = focusedComponent.getActionForKeyStroke(keyStroke);

			//Determine whether the currently focussed component can handle the event
			if (focusedComponent instanceof JTextComponent || actionListener != null)
			{
				keyEvent = new KeyEvent(focusedComponent, KeyEvent.KEY_PRESSED, (new Date()).getTime(), modifierMask, keyCode);
				focusedComponent.dispatchEvent(keyEvent);
			}
		}//*/
	}

	/**
	 * This will be called when the user selects undo or redo from
	 * the menu or presses the appropriate hot keys. It will
	 * tell the current undo manager to undo redo depending on the
	 * und ovalue given.
	 *
	 * @param	undo	True to undo, false to redo
	 */
	protected void doUndoRedo(boolean undo)
	{
		if (this.currentUndoManager != null)
		{
			if (undo)
			{
				this.currentUndoManager.undo();
			}
			else
			{
				this.currentUndoManager.redo();
			}
			
			this.setUndoItems();
		}
		else
		{
			System.out.println("Undo manager is null");
		}
	}

	/**
	 * This will handle program flow for the menu items
	 */
	public class BActionListener implements ActionListener
	{
		/**
		 * Called when the user selects a menu item
		 */
		public void actionPerformed(ActionEvent actionEvent)
		{
			Component focusedComponent = BWindowManager.getFocusOwner();
			String command = actionEvent.getActionCommand();
			Object source = actionEvent.getSource();
			boolean internalEvent = false;

			if (focusedComponent instanceof JComponent)
			{
                if (undoManagers.containsKey(focusedComponent))
                {
                    setUndoManager((BUndoManager)undoManagers.get(focusedComponent));
                }

				if (source == copyMenuItem || source == copyButton)
				{
					dispatchEvent((JComponent)focusedComponent, KeyEvent.VK_C, InputEvent.CTRL_MASK);
				}
				else if (source == cutMenuItem || source == cutButton)
				{
					dispatchEvent((JComponent)focusedComponent, KeyEvent.VK_X, InputEvent.CTRL_MASK);
				}
				else if (source == pasteMenuItem || source == pasteButton)
				{
					dispatchEvent((JComponent)focusedComponent, KeyEvent.VK_V, InputEvent.CTRL_MASK);
				}
                else if (source == deleteMenuItem)
				{
					dispatchEvent((JComponent)focusedComponent, KeyEvent.VK_DELETE, 0);
				}
                else if (source == selectAllMenuItem)
				{
					dispatchEvent((JComponent)focusedComponent, KeyEvent.VK_A, InputEvent.CTRL_MASK);
				}
				else if (source == undoMenuItem || source == undoButton)
				{
					doUndoRedo(true);
				}
				else if (source == redoMenuItem || source == redoButton)
				{
					doUndoRedo(false);
				}
			}
		}
	}
}
