

package au.edu.swin.synergy.plugin.packager;

import au.edu.swin.synergy.plugin.metamodeller.BMetaElement;
import au.edu.swin.synergy.plugin.diagram.BDrawable;

import au.edu.swin.synergy.debug.Debug;

import javax.swing.tree.TreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultMutableTreeNode;

import java.util.Enumeration;
import java.util.Vector;
import java.io.Serializable;

/**
 * This will contain a MetaElement and allow a means of
  * editting the way this looks as well as displaying it
 * as a hierarchy in a tree.
 *
 * @author Tristan Austin
 */
public class BPackageModelElement implements MutableTreeNode, Serializable
{
    /**
     * Indicates whether this element will allow children
     */
    private static final boolean kAllowsChildren = true;

    /**
     * This is the meta element that this tree node represents
     */
    private BMetaElement metaElement = null;

    /**
     * The properties of this element represent its children
     */
    private Vector propertyNodes = new Vector();

    /**
     * The parent node of this node
     */
    private MutableTreeNode parent = null;

    /**
     * This is the drawable that represents the BMetaElement in this package
     */
    private BDrawable packageDrawable = null;

    /**
     * Indicates that this is a package node
     */
    public static final int kPackageNode = 1;

    /**
     * Indicates that this is a package arc
     */
    public static final int kPackageArc = 2;

    /**
     * Indicates that this is a package terminator
     */
    public static final int kPackageTerminator = 3;

    /**
     * This is the type of node that this element represents
     */
    private int nodeType = kPackageNode;

    /**
     * Creates a new BPackageModelElement based on the given
     * meta element.
     *
     * @param   metaElement The element to base this on
     * @param   nodeType    The type of node creating, BPackageArc, BPackageNode, BPackageTerminator
     */
    public BPackageModelElement(BMetaElement metaElement, int nodeType)
    {
        this.nodeType = nodeType;
				
        //Create the group that will represent this BMetaElement
        if (nodeType == kPackageArc)
        {
            //this.packageGroup = new BPackageArc()
        }
        else if (nodeType == kPackageNode)
        {
            this.packageDrawable = new BPackageNode( metaElement);
        }
        else if (nodeType == kPackageTerminator)
        {
            //this.packageGroup = new BPackageTerminator();
        }

		this.setUserObject(metaElement);        
    }

    /**
     * Returns the child TreeNode at index childIndex.
     */
    public TreeNode getChildAt(int childIndex)
    {
        TreeNode childAt = null;

        //Get the child at that location
        childAt = (TreeNode)this.propertyNodes.elementAt(childIndex);

        //Return the child
        return childAt;
    }

    /**
     * Returns the number of children TreeNodes the receiver contains.
     *
     * @return The number of children of this node
     */
    public int getChildCount()
    {
        int count = 0;

        //Get the number of propertyNodes
        count = this.propertyNodes.size();

        //Return the number of propertyNodes
        return count;
    }

    /**
     * Returns the parent TreeNode of the receiver.
     *
     * @return The parent tree node of this tree node
     */
    public TreeNode getParent()
    {
        return this.parent;
    }

    /**
     * Returns the index of node in the receivers children. If the
     * receiver does not contain node, -1 will be returned.
     */
    public int getIndex(TreeNode node)
    {
        int location = -1;

        //Get the index of it
        location = this.propertyNodes.indexOf(node);

        //Return the location
        return location;
    }

    /**
     * Returns true if the receiver allows children. This will
     * always return true because this class only represents
     * an element with properties.
     *
     * @return True always
     */
    public boolean getAllowsChildren()
    {
        return BPackageModelElement.kAllowsChildren;
    }

    /**
     * Returns true if the receiver is a leaf. This will return
     * true when it has no children
     *
     * @return True if there are no children, false otherwise
     */
    public boolean isLeaf()
    {
        //Is only a leaf if there are no children
        return ((this.propertyNodes == null) || (this.propertyNodes.size() == 0));
    }

    /**
     * Returns the children of the reciever as an Enumeration.
     */
    public Enumeration children()
    {
        Enumeration children = null;

        //Enumerate the children
        children = this.propertyNodes.elements();

        //Return the enumeration
        return children;
    }

    /**
     * This will get the name of the meta element
     * as the graphical representation on the screen
     *
     * @return The name of the element
     */
    public String toString()
    {
        return this.metaElement.getName();
    }

    /**
     * Provides a reference to the meta element that
     * this node represents.
     *
     * @return The meta element this node represents
     */
    public Object getUserObject()
    {
        return this.metaElement;
    }

    /**
     * This allows the user to
     */
    public void setUserObject(Object metaElementObject)
    {
        BMetaElement metaElement = (BMetaElement)metaElementObject;
        Vector properties = metaElement.getProperties();
        this.metaElement = metaElement;

        //If there are any properties
        if (properties != null)
        {
            //Run through them and create nodes based on them
            for (int i = 0; i < properties.size(); i++)
            {
                this.propertyNodes.addElement(new DefaultMutableTreeNode(properties.elementAt(i), false));
            }
        }
		//set the groups name to that of the metaitems name
		this.packageDrawable.setName( this.metaElement.getName() );

    }

    /**
     * Sets the parent node of this node to the given
     * tree node.
     *
     * @param   parentTreeNode  The new parent of this node
     */
    public void setParent(MutableTreeNode parentTreeNode)
    {
        this.parent = parentTreeNode;
    }

    /**
     * This will remove this node from its parent node by calling
     * the remove method on the parent object and passing
     * this as the paramater to the call.
     *
     * @pre This node has a parent
     * @post    This node no longer has a parent
     */
    public void removeFromParent()
    {
        this.parent.remove(this);
    }

    /**
     * Removes the given node from the list of children
     * of this node. This probably won't be used as the
     * hierarchy of the package modeller is fairly static.
     *
     * @param   targetNode  The node to remove
     */
    public void remove(MutableTreeNode targetNode)
    {
        BPackageModelElement targetElement = (BPackageModelElement)targetNode;
        String targetProperty = (String)targetElement.getUserObject();
        this.propertyNodes.remove(targetNode);
        this.metaElement.removeProperty(targetProperty);
    }

    /**
     * Removes the given node from the list of children
     * of this node. This probably won't be used as the
     * hierarchy of the package modeller is fairly static.
     *
     * @param   targetNode  The node to remove
     */
    public void remove(int targetNodeIndex)
    {
        this.propertyNodes.removeElementAt(targetNodeIndex);
        this.metaElement.removePropertyAt(targetNodeIndex);
    }

    /**
     * This will insert the given MutableTreeNode into
     * the element at the specified index.
     *
     * @param   newNode The node to insert
     * @param   index   The index to insert the new node
     */
    public void insert(MutableTreeNode newNode, int index)
    {
        BPackageModelElement targetNode = (BPackageModelElement)newNode;
        this.propertyNodes.insertElementAt(newNode, index);
        this.metaElement.insertPropertyAt((String)targetNode.getUserObject(), index);
    }

    /**
     * This will return the BGroup that represents this
     * BMetaElement in this package.
     *
     * @return The BGroup representing this meta element
     */
    public BDrawable getBDrawable()
    {
        return this.packageDrawable;
    }

    /**
     * This will indicate the type of node this node
     * represents.
     *
     * @return kPackageArc, kPackageNode, kPackageTerminator
     */
    public int getNodeType()
    {
        return this.nodeType;
    }
}
