

package au.edu.swin.synergy.plugin.userdb;

import au.edu.swin.synergy.kernel.BCommandComplete;
import au.edu.swin.synergy.kernel.BCommandEvent;
import au.edu.swin.synergy.kernel.BPlugin;

import au.edu.swin.synergy.lib.UnixCrypt;

import au.edu.swin.synergy.debug.Debug;

import au.edu.swin.synergy.plugin.userdb.event.BChangePasswordCE;
import au.edu.swin.synergy.plugin.userdb.event.BAuthUserCE;

import au.edu.swin.synergy.plugin.server.event.BGetUserCE;
import au.edu.swin.synergy.plugin.server.event.BGetSessionsCE;
import au.edu.swin.synergy.plugin.server.BServerSession;

import au.edu.swin.synergy.kernel.BBlockingComplete;

import java.util.Hashtable;
import java.util.Vector;

/**
 * This handles all security relating to passwords,
 * privileges and owners. It also does the changing
 * of passwords as this is a particular specialised
 * functionality.
 *
 * @author Tristan Austin
 */
public class BSecurityManager
{
    /**
     * This is a reference to the hashtable containing the list of users
     */
    private Hashtable userDatabase;

    /**
     * This is a reference to the hashtable containing the list of users
     */
    private Hashtable groupDatabase;

    /**
     * This is used to maintain the cross references between users and groups
     */
    private BReferentialIntegrity referentialManager = new BReferentialIntegrity();

    /**
     * This is the plugin that we will use to dispatch events
     */
    private BPlugin eventDispatcher;

    /**
     * Creates a new security manager with the given users and groups
     * to manage.
     *
     * @param   userDatabase The users of the system
     * @param   groupDatabase   The groups in the system
     */
    public BSecurityManager(BPlugin dispatcher, Hashtable userDatabase, Hashtable groupDatabase)
    {
        this.userDatabase = userDatabase;
        this.groupDatabase = groupDatabase;
        this.eventDispatcher = dispatcher;
    }

    /**
     * This will take the change password event, find the user
     * that is intended to be updated, compare the original
     * password with the existing one and update the password
     * if all is well. If not, it will indicate such in the
     * completion object and set the reason why in the event itself.
     *
     * @param passwordEvent   The event requesting the change
     * Please complete the missing tags for doPasswordChange
     * @return
     * @pre
     * @post
     */
    protected void doPasswordChange( BChangePasswordCE passwordEvent )
    {
        String userName = ((BUser)passwordEvent.getItem()).getName();
        String oldPassword = passwordEvent.getOldPassword();
        String newPassword = passwordEvent.getNewPassword();
        BUser user = null;

        //First find the user
        user = ( BUser ) this.userDatabase.get( userName );
        
        //Make sure they exist
        if( user != null )
        {
            //See if the password matches
            if( this.isAdministrator(passwordEvent) || this.doPasswordCheck( oldPassword, user.getPassword() ) )
            {
                //Set the new password
                user.setPassword( this.doPasswordEncrypt( newPassword ) );
                passwordEvent.setState( BChangePasswordCE.kChanged );
                passwordEvent.doComplete( BCommandComplete.kSuccess );
                Debug.doPrintln( "Password change for " + user.getName() + " succesfull" );
            }
            else
            {
                //Indicate that the password was not correct
                passwordEvent.setState( BChangePasswordCE.kWrongPassword );
                passwordEvent.doComplete( BCommandComplete.kError );
                Debug.doPrintln( "Incorrect password" );
            }
        }
        else
        {
            //Indicate that the user wasn't found in the database
            passwordEvent.setState( BChangePasswordCE.kNoUser );
            passwordEvent.doComplete( BCommandComplete.kError );
            Debug.doPrintln( "User not found" );
        }
    }

    /**
     * This will take a plain text password and see if
     * it matches the encrypted password provided.
     *
     * @param   plainPassword   The plain text password
     * @param   encryptedPassword   the encrypted password
     * @return True if they match, false otherwise
     * Please complete the missing tags for doPasswordCheck
     * @pre
     * @post
     */
    private boolean doPasswordCheck( String plainPassword, String encryptedPassword )
    {
        return UnixCrypt.matches( encryptedPassword, plainPassword );
    }

    /**
     * This will take the given plain password
     * and return the encrypted version of it
     *
     * @param plainPassword The password to encrypt
     * @return The encrypted version of the password
     * Please complete the missing tags for doPasswordEncrypt
     * @pre
     * @post
     */
    protected String doPasswordEncrypt( String plainPassword )
    {
        return UnixCrypt.crypt( plainPassword );
    }


    /**
     * This will determine whether the given credentials
     * are valid by comparing them with the credentials
     * in the user database.
     *
     * @param   e   The authenticate user event
     */
    public void doAuthentication(BAuthUserCE e)
    {
        Debug.doPrintln("\n\nBAuthUserCE\n\n");

        // assume failure
        e.setStatus( BAuthUserCE.kFail );

        BUser user = ( BUser ) userDatabase.get( e.getCredentials().getUsername() );
        if( user != null )
        {
            Debug.doPrintln("Checking password for: " + user.getName());
            Debug.doPrintln("Password: " + user.getPassword());
            Debug.doPrintln("Credentials is: " + e.getCredentials());
            if( this.doPasswordCheck( e.getCredentials().getPassword(), user.getPassword() ) )
            {
                e.setStatus( BAuthUserCE.kSuccess );
                e.setUser( user );

                // resolve the group names into numbers
                this.referentialManager.resolveGroupNumbers(this.groupDatabase, user);
            }
        }

        e.doComplete( BCommandComplete.kSuccess );
    }

    /**
     * This will determine whether the user currently
     * authenticated against this server has administrative
     * privileges or not.
     *
     * @param   targetUser  The user object being updated
     * @return True if they are, false otherwise
     */
    protected boolean isAdministrator(BCommandEvent event)
    {
        Vector groups;
        BUser currentUser;
        String currentGroupName;
        BGroup currentGroup;
        boolean isAdmin = false;
        boolean isLocalEvent = this.isLocal(event);

        //If not local
        if (!isLocalEvent)
        {
            currentUser = this.getAuthenticatedUser(event.getConnectionID());
            isAdmin = (currentUser.getPrivilege() == BPrivileged.kAdministrator);

            //Check the groups this user is a member of
            if (!isAdmin)
            {
                groups = currentUser.getMembers();

                //Run through all the groups and see if any of them have admin privileges
                for (int i = 0; i < groups.size(); i++)
                {
                    currentGroupName = (String)groups.elementAt(i);
                    currentGroup = (BGroup)this.groupDatabase.get(currentGroupName);

                    //See if this group has admin privileges
                    isAdmin = (currentGroup.getPrivilege() == BPrivileged.kAdministrator);

                    //If we find an admin group, return
                    if (isAdmin)
                    {
                        break;
                    }
                }
            }
        }

        //User is to be treated as admin if the event is local or they are admin
        return (isLocalEvent || isAdmin);
    }

    /**
     * This will determine whether the given event originated
     * locally or originated from a source other than the local machine.
     *
     * @param   event   The event to check
     */
    public boolean isLocal(BCommandEvent event)
    {
        //See if the event came from the local kernel
        return (event.getConnectionID() == -1);
    }

    /**
     * This will determine whether the user currently
     * authenticated against this server attempting to update
     * the provided user object is infact the owner of the
     * object.
     *
     * @param   targetUser  The user object being updated
     * @return True if they are, false otherwise
     */
    protected boolean isOwner(BUser targetUser, int connectionID)
    {
        BUser currentUser = this.getAuthenticatedUser(connectionID);

        //Indicate whether the current user owns the target user
        return (currentUser.equals(targetUser));
    }

    /**
     * This will get the user currently
     * authenticated against this server
     *
     * @returns The user requesting these services
     */
    BUser getAuthenticatedUser(int connectionID)
    {
        BBlockingComplete onGetUser = new BBlockingComplete();
        BGetUserCE getUserEvent = new BGetUserCE(this, connectionID, onGetUser);

        //Dispatch the request
        this.eventDispatcher.dispatchEvent(getUserEvent);

        //Wait for the response
        onGetUser.waitForEvent();

        //Get the current user and return it
        getUserEvent = (BGetUserCE)onGetUser.getEvent();
        return getUserEvent.getUser();
    }

    /**
     * Indicates whether the given user is currently logged in
     *
     * @param   user    The user to check
     * @return True if they are logged in, false otherwise
     */
    public boolean isLoggedIn(BUser user)
    {
        BGetSessionsCE getSessions = new BGetSessionsCE(this, null);
        BServerSession session;
        BUser currentUser;
        this.eventDispatcher.dispatchEventAndWait(getSessions);
        int count = getSessions.getCount();

        //Find the user
        for (int i = 0; i < count; i++)
        {
            session = getSessions.getSession(i);
            currentUser = session.getUser();

            //If we have a match, return true
            if (currentUser.equals(user))
            {
                return true;
            }
        }

        //Didn't find one
        return false;
    }
}
