

package au.edu.swin.synergy.plugin.userman;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.lib.BInputReader;

import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.plugin.userdb.event.BAddGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateGroupCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveGroupCE;
import au.edu.swin.synergy.plugin.userdb.BInvalidPropertyException;

import au.edu.swin.synergy.kernel.BPlugin;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.plugin.dialect.BDialect;

import au.edu.swin.synergy.debug.Debug;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;

import java.util.Vector;

/**
 * The add group command is a command line registration of
 * the add group command. It will create a BAddUserCE event
 * and dispatch it.
 *
 * @author John Newbigin
 */
   class BGroupManCL implements BOnCommand
   {
    /**
     * The plugin used to dispatch events via
     */
      private BUserMan userManager;

     /**
      * These are the available groups the user may choose from
      */
      private Vector availableUsers = new Vector();

     /**
      * These are the available groups the user may choose from
      */
      private Vector availableGroups = new Vector();

      /**
       * This provides common utilities to interact with the command line
       */
      private BUserManCLTools clTools;

    /**
     * Sets the available groups. This is expected to be
     * called whenever the list is updated in the user
     * manager.
     *
     * @param   groups  The updated list of available groups
     */
     public void setAvailableGroups(Vector groups)
     {
        //Make sure we synchronize on the available groups
        synchronized(this.availableGroups)
        {
            //Once these are set once, doesn't need to be done again
            this.clTools.setInitialised(true);

            //Update the existing groups
            this.availableGroups.removeAllElements();
            this.availableGroups.addAll(groups);
        }
     }

     /**
     * Sets the available users. This is expected to be
     * called whenever the list is updated in the user
     * manager.
     *
     * @param   users  The updated list of available users
     */
     public void setAvailableUsers(Vector users)
     {
        //Make sure we synchronize on the available groups
        synchronized(this.availableUsers)
        {
            //Once these are set once, doesn't need to be done again
            this.clTools.setInitialised(true);

            //Update the existing groups
            this.availableUsers.removeAllElements();
            this.availableUsers.addAll(users);
        }
     }

     /**
      * Creates a new command line class with the given plugin
      * to dispatch events via. This list of available users and
      * groups is expected to be updated by the user manager. And
      * the doRefresh() command in the usermanager will be called
      * if no users or groups are available on the first run.
      *
      * @param  userManager  The plugin to dispatch events via
      */
     public BGroupManCL(BUserMan userManager)
     {
        this.userManager = userManager;

        //Create the tools object
        this.clTools = new BUserManCLTools(this.userManager);
     }

    /**
     * When the command line argument associated
     * with this object is entered by the user, this
     * method is called. The trailing arguments
     * are passed in as paramaters in a BStringList
     *
     * @param   params  A BStringList of Strings containing the paramaters
     */
      public int onCommand( BStringList params, InputStream in, PrintStream out )
      {
        boolean isAdministrator = false;

         //need to be logged in to do this
        //This will make sure we have everything
        this.clTools.doInitialisation();

        try
        {
             //Make sure there's enough paramaters
             if( params.getCount() >= 2 )
             {
                if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "addcmd")))
                {
                    //Check to see the user is an administrator
                    if (this.clTools.doAdminCheck(true))
                    {
                        this.doAddGroup(in, out);
                    }
                }
                else if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "editcmd")))
                {
                    //Make sure they have specified what to edit
                    if (params.getCount() == 3)
                    {
                        //Make sure they're allowed to edit
                        if ((isAdministrator = this.clTools.doAdminCheck(false)) || this.clTools.doOwnerCheck(params.get(2), false))
                        {
                            this.doUpdateGroup(params.get(2), isAdministrator, in, out);
                        }
                        else
                        {
                            this.clTools.doMessageSend(BDialect.getString(this, "noadmin"), BStandardOutCE.kError);
                        }
                    }
                    else
                    {
                        //Tell the user the input was wrong
                        this.clTools.doMessageSend(BDialect.getString(this, "toedit"), BStandardOutCE.kWarning);
                    }
                }
                else if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "showcmd")))
                {
                    //Determine whether to show all or just a group
                    //Make sure they have specified what to edit
                    if (params.getCount() == 3)
                    {
                        //Determine what they want to see
                        if (params.get(2).equalsIgnoreCase(BDialect.getString(this, "allcmd")))
                        {
                            //Show all the groups
                            clTools.doShowAll(this.availableGroups, out);
                        }
                        else
                        {
                            //Show the specified group
                            this.doShowGroup(params.get(2), out);
                        }
                    }
                    else
                    {
                        //Show all the groups
                        clTools.doShowAll(this.availableGroups, out);
                    }
                }
                else if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "removecmd")))
                {
                    //Make sure they're allowed to do this
                    if (this.clTools.doAdminCheck(true))
                    {
                        this.doRemoveGroup(params.get(2), in, out);
                    }
                }
                else
                {
                    //Show them how it's done
                    out.println(this.getHelp(params));
                }
             }
             else
             {
                //Show all the groups
                clTools.doShowAll(this.availableGroups, out);
             }
            }
            catch(IOException ioException)
            {
                ioException.printStackTrace();
            }//*/

        //Indicate handled
        return 0;
    }

    /**
     * This will remove the specified group from the database.
     *
     * @param   groupName    The group to remove
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     */
    private void doRemoveGroup(String groupName, InputStream in, PrintStream out)
    {
        BGroup targetGroup = (BGroup)this.clTools.getPrivileged(groupName, this.availableGroups);
        BRemoveGroupCE removeGroup;

        //See if the group exists
        if (targetGroup != null)
        {
            removeGroup = new BRemoveGroupCE(this, targetGroup, new BServerUpdateCompleteCC(this.userManager));

            //Dispatch the request
            this.userManager.dispatchEventServer(removeGroup);
        }
        else
        {
            this.clTools.doMessageSend(BDialect.getString(this, "nogroup"), BStandardOutCE.kError);
        }
    }

    /**
     * When called, this method will read all the information required
     * from the group and send the add group request to the server.
     *
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     */
    private void doUpdateGroup(String editGroup, boolean isAdmin, InputStream in, PrintStream out) throws IOException
    {
        //Create the group and populate the group
        BGroup group;

        group = (BGroup)this.clTools.getPrivileged(editGroup, this.availableGroups);

        //Make sure the group exists
        if (group != null)
        {
            this.doGroupInput(group, in, out, true, isAdmin);

            //Make sure the group is ok
            if (this.clTools.doConfirm(BDialect.getString(this, "proceedq"), in, out))
            {
                //Create the event and dispatch to server
               BUpdateGroupCE event = new BUpdateGroupCE( this, group, new BServerUpdateCompleteCC(this.userManager) );
               this.userManager.dispatchEventServer( event );
            }
        }
        else
        {
            //Notify the user that the given group wasn't found
            this.clTools.doMessageSend(BDialect.getString(this, "groupnf"), BStandardOutCE.kError);
        }
    }

    /**
     * When called, this method will read all the information required
     * from the user and send the add group request to the server.
     *
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     */
    private void doAddGroup(InputStream in, PrintStream out) throws IOException
    {
        //Create the group and populate the group
        BGroup group = new BGroup();
        this.doGroupInput(group, in, out);

        //Make sure the group is ok
        if (this.clTools.doConfirm(BDialect.getString(this, "proceedq"), in, out))
        {
            //Create the event and dispatch to server
           BAddGroupCE event = new BAddGroupCE( this, group, new BServerUpdateCompleteCC(this.userManager) );
           this.userManager.dispatchEventServer( event );
        }
    }

    /**
     * This will locate the group with the given name and display
     * their details. If the group isn't found, this fact
     * will be displayed.
     *
     * @param   groupName    the name of the group to display
     * @param   out The output stream to write to
     */
    private void doShowGroup(String groupName, PrintStream out)
    {
        BGroup group = (BGroup)this.clTools.getPrivileged(groupName, this.availableGroups);
        String col = BDialect.getString(this, "separator") + " ";
        
        if (group != null)
        {
            out.println(BDialect.getString(this, "name") + col + group.getName());
            out.println(BDialect.getString(this, "privilege") + col + this.clTools.getPrivilege(group));
            out.println(BDialect.getString(this, "users") + col + group.getMembers());
            out.println(BDialect.getString(this, "comments") + col + group.getComments());
        }
        else
        {
            this.clTools.doMessageSend(BDialect.getString(this, "groupnf"), BStandardOutCE.kError);
        }
    }


    /**
     * This will get all the details from the group and add the new
     * group to the database.
     *
     * @param   Group    the group to populate with values
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @param   edit    Indicates whether we're editting or adding True for editting false for adding
     */
    private void doGroupInput(BGroup group, InputStream in, PrintStream out) throws IOException
    {
        this.doGroupInput(group, in, out, false, true);
    }

    /**
     * This will get all the details from the user
     *
     * @param   Group    the group to populate with values
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @param   edit    Indicates whether we're editting or adding True for editting false for adding
     * @param   isOwner Indicates whether the user is editting their own group object
     */
    private void doGroupInput(BGroup group, InputStream in, PrintStream out, boolean edit, boolean isAdmin) throws IOException
    {
        int privilege = BPrivileged.kUser;
        boolean getValue = isAdmin;
        String col = BDialect.getString(this, "separator") + " ";

        try
        {
            //Indicate current value for editting
            if (edit && isAdmin)
            {
                //Determine whether they want to edit it
                out.println(BDialect.getString(this, "name") + col + group.getName());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue && isAdmin)
            {
                //Get the group name
                group.setName(clTools.doGetString(BDialect.getString(this, "name") + col, true, in, out));
                getValue = true;
            }


            //Indicate current value for editting
            if (edit && isAdmin)
            {
                out.println(BDialect.getString(this, "privilege") + col + this.clTools.getPrivilege(group));
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue && isAdmin)
            {
                //Get the privilege of the group
                group.setPrivilege(clTools.doGetInt(BDialect.formatText(BDialect.getString(this, "privilegeprompt"), new Object[]{new Integer(BPrivileged.kUser), new Integer(BPrivileged.kAdministrator)}) + BPrivileged.kAdministrator + "-Administrator) ", in, out, true, BPrivileged.kUser, BPrivileged.kAdministrator));
                getValue = true;
            }

            //Indicate current value for editting
            if (edit && isAdmin)
            {
                out.println(BDialect.getString(this, "groups") + col + group.getMembers());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue && isAdmin)
            {
                //Make sure we don't have conflict
                synchronized(this.availableUsers)
                {
                    //Get the groups of this new group
                    out.println(BDialect.getString(this, "groups") + col);
                    group.setMembers(this.clTools.getMembers(this.availableUsers, in, out));
                    getValue = true;
                }
            }

            getValue = true;

            //Indicate current value for editting
            if (edit)
            {
                out.println(BDialect.getString(this, "comments") + col + group.getComments());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue)
            {
                //Get the comments
                group.setComments(clTools.doGetString(BDialect.getString(this, "comments") + col, false, in, out));
            }
        }
        catch(BInvalidPropertyException invalidException)
        {
        }
    }

    /**
     * When the user types "help" at the command prompt, the kernel
     * will call the getHelp method on all the registered onCommand
     * objects. This will indicate how to use this command line option.
     *
     * @param   params  The paramatesr to the help call (eg. help naddgroup)
     */
      public String getHelp( BStringList params )
      {
        return BDialect.getString(this, "groupclhelp");
      }
   }




