

package au.edu.swin.synergy.plugin.userman;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.lib.BInputReader;

import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.plugin.userdb.event.BAddUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveUserCE;
import au.edu.swin.synergy.plugin.userdb.BInvalidPropertyException;

import au.edu.swin.synergy.kernel.BPlugin;

import au.edu.swin.synergy.plugin.dialect.BDialect;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.debug.Debug;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;

import java.util.Vector;

/**
 * The add user command is a command line registration of
 * the add user command. It will create a BAddUserCE event
 * and dispatch it.
 *
 * @author John Newbigin
 */
   class BUserManCL implements BOnCommand
   {
    /**
     * The plugin used to dispatch events via
     */
      private BUserMan userManager;

     /**
      * These are the available groups the user may choose from
      */
      private Vector availableGroups = new Vector();

     /**
      * These are the available groups the user may choose from
      */
      private Vector availableUsers = new Vector();

      /**
       * This provides common utilities to interact with the command line
       */
      private BUserManCLTools clTools;

    /**
     * Sets the available groups. This is expected to be
     * called whenever the list is updated in the user
     * manager.
     *
     * @param   groups  The updated list of available groups
     */
     public void setAvailableGroups(Vector groups)
     {
        //Make sure we synchronize on the available groups
        synchronized(this.availableGroups)
        {
            //Once these are set once, doesn't need to be done again
            this.clTools.setInitialised(true);

            //Update the existing groups
            this.availableGroups.removeAllElements();
            this.availableGroups.addAll(groups);
        }
     }

     /**
     * Sets the available users. This is expected to be
     * called whenever the list is updated in the user
     * manager.
     *
     * @param   users  The updated list of available users
     */
     public void setAvailableUsers(Vector users)
     {
        //Make sure we synchronize on the available groups
        synchronized(this.availableUsers)
        {
            //Once these are set once, doesn't need to be done again
            this.clTools.setInitialised(true);

            //Update the existing groups
            this.availableUsers.removeAllElements();
            this.availableUsers.addAll(users);
        }
     }

     /**
      * Creates a new command line class with the given plugin
      * to dispatch events via. This list of available users and
      * groups is expected to be updated by the user manager. And
      * the doRefresh() command in the usermanager will be called
      * if no users or groups are available on the first run.
      *
      * @param  userManager  The plugin to dispatch events via
      */
     public BUserManCL(BUserMan userManager)
     {
        this.userManager = userManager;

        //Create the tools object
        this.clTools = new BUserManCLTools(this.userManager);
     }

    /**
     * When the command line argument associated
     * with this object is entered by the user, this
     * method is called. The trailing arguments
     * are passed in as paramaters in a BStringList
     *
     * @param   params  A BStringList of Strings containing the paramaters
     */
      public int onCommand( BStringList params, InputStream in, PrintStream out )
      {
        boolean isAdministrator = false;

         //need to be logged in to do this
        //This will make sure we have everything
        this.clTools.doInitialisation();

        try
        {
             //Make sure there's enough paramaters
             if( params.getCount() >= 2 )
             {
                if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "addcmd")))
                {
                    //Check to see the user is an administrator
                    if (this.clTools.doAdminCheck(true))
                    {
                        this.doAddUser(in, out);
                    }
                }
                else if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "editcmd")))
                {
                    //Make sure they have specified what to edit
                    if (params.getCount() == 3)
                    {
                        //Make sure they're allowed to edit
                        if ((isAdministrator = this.clTools.doAdminCheck(false)) || this.clTools.doOwnerCheck(params.get(2), false))
                        {
                            this.doUpdateUser(params.get(2), isAdministrator, in, out);
                        }
                        else
                        {
                            this.clTools.doMessageSend(BDialect.getString(this, "needadmin"), BStandardOutCE.kError);
                        }
                    }
                    else
                    {
                        //Tell the user the input was wrong
                        this.clTools.doMessageSend(BDialect.getString(this, "toedit"), BStandardOutCE.kWarning);
                    }
                }
                else if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "showcmd")))
                {
                    //Determine whether to show all or just a user
                    //Make sure they have specified what to edit
                    if (params.getCount() == 3)
                    {
                        //Determine what they want to see
                        if (params.get(2).equalsIgnoreCase(BDialect.getString(this, "allcmd")))
                        {
                            //Show all the users
                            clTools.doShowAll(this.availableUsers, out);
                        }
                        else
                        {
                            //Show the specified user
                            this.doShowUser(params.get(2), out);
                        }
                    }
                    else
                    {
                        //Show all the users
                        clTools.doShowAll(this.availableUsers, out);
                    }
                }
                else if (params.get(1).equalsIgnoreCase(BDialect.getString(this, "removecmd")))
                {
                    //Make sure they're allowed to do this
                    if (this.clTools.doAdminCheck(true))
                    {
                        this.doRemoveUser(params.get(2), in, out);
                    }
                }
                else
                {
                    //Show them how it's done
                    this.clTools.doMessageSend(this.getHelp(params), BStandardOutCE.kWarning);
                }
             }
             else
             {
                //Show all the users
                clTools.doShowAll(this.availableUsers, out);
             }
        }
        catch(IOException ioException)
        {
            ioException.printStackTrace();
        }//*/

        //Indicate handled
        return 0;
    }

    /**
     * This will remove the specified user from the database.
     *
     * @param   userName    The user to remove
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     */
    private void doRemoveUser(String userName, InputStream in, PrintStream out)
    {
        BUser targetUser = (BUser)this.clTools.getPrivileged(userName, this.availableUsers);
        BRemoveUserCE removeUser;

        //See if the user exists
        if (targetUser != null)
        {
            removeUser = new BRemoveUserCE(this, targetUser, new BServerUpdateCompleteCC(this.userManager));

            //Dispatch the request
            this.userManager.dispatchEventServer(removeUser);
        }
        else
        {
            this.clTools.doMessageSend(BDialect.getString(this, "nouser"), BStandardOutCE.kError);
        }
    }

    /**
     * When called, this method will read all the information required
     * from the user and send the add user request to the server.
     *
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     */
    private void doUpdateUser(String editUser, boolean isAdmin, InputStream in, PrintStream out) throws IOException
    {
        //Create the user and populate the user
        BUser user;

        user = (BUser)this.clTools.getPrivileged(editUser, this.availableUsers);

        //Make sure the user exists
        if (user != null)
        {
            this.doUserInput(user, in, out, true, isAdmin);

            //Make sure the user is ok
            if (this.clTools.doConfirm(BDialect.getString("proceedq"), in, out))
            {
                //Create the event and dispatch to server
               BUpdateUserCE event = new BUpdateUserCE( this, user, new BServerUpdateCompleteCC(this.userManager) );
               this.userManager.dispatchEventServer( event );
            }
        }
        else
        {
            //Notify the user that the given user wasn't found
            this.clTools.doMessageSend(BDialect.getString(this, "usernf"), BStandardOutCE.kError);
        }
    }

    /**
     * When called, this method will read all the information required
     * from the user and send the add user request to the server.
     *
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     */
    private void doAddUser(InputStream in, PrintStream out) throws IOException
    {
        //Create the user and populate the user
        BUser user = new BUser();
        this.doUserInput(user, in, out);

        //Make sure the user is ok
        if (this.clTools.doConfirm(BDialect.getString("proceedq"), in, out))
        {
            //Create the event and dispatch to server
           BAddUserCE event = new BAddUserCE( this, user, user.getPassword(), new BServerUpdateCompleteCC(this.userManager) );
           this.userManager.dispatchEventServer( event );
        }
    }

    /**
     * This will locate the user with the given name and display
     * their details. If the user isn't found, this fact
     * will be displayed.
     *
     * @param   userName    the name of the user to display
     * @param   out The output stream to write to
     */
    private void doShowUser(String userName, PrintStream out)
    {
        BUser user = (BUser)this.clTools.getPrivileged(userName, this.availableUsers);
        String col = BDialect.getString(this, "separator") + " ";

        if (user != null)
        {
            out.println(BDialect.getString(this, "username") + col + user.getName());
            out.println(BDialect.getString(this, "fullname") + col + user.getFullName());
            out.println(BDialect.getString(this, "email") + col + user.getEmailAddress());
            out.println(BDialect.getString(this, "alias") + col + user.getAlias());
            out.println(BDialect.getString(this, "privilege") + col + this.clTools.getPrivilege(user));
            out.println(BDialect.getString(this, "groups") + col + user.getMembers());
            out.println(BDialect.getString(this, "comments") + col + user.getComments());
        }
        else
        {
            this.clTools.doMessageSend(BDialect.getString(this, "nouser"), BStandardOutCE.kError);
        }
    }


    /**
     * This will get all the details from the user and add the new
     * user to the database.
     *
     * @param   User    the user to populate with values
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @param   edit    Indicates whether we're editting or adding True for editting false for adding
     */
    private void doUserInput(BUser user, InputStream in, PrintStream out) throws IOException
    {
        this.doUserInput(user, in, out, false, true);
    }

    /**
     * This will get all the details from the user
     *
     * @param   User    the user to populate with values
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @param   edit    Indicates whether we're editting or adding True for editting false for adding
     * @param   isOwner Indicates whether the user is editting their own user object
     */
    private void doUserInput(BUser user, InputStream in, PrintStream out, boolean edit, boolean isAdmin) throws IOException
    {
        int privilege = BPrivileged.kUser;
        boolean getValue = isAdmin;
        String col = BDialect.getString(this, "separator") + " ";

        try
        {
            //Indicate current value for editting
            if (edit && isAdmin)
            {
                //Determine whether they want to edit it
                out.println(BDialect.getString(this, "username") + col + user.getName());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue && isAdmin)
            {
                //Get the user name
                user.setName(clTools.doGetString(BDialect.getString(this, "username") + col , true, in, out));
                getValue = true;
            }

            //Indicate current value for editting
            if (edit)
            {
                out.println(BDialect.getString(this, "fullname") + col + user.getFullName());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue)
            {
                //Get the full name
                user.setFullName(clTools.doGetString(BDialect.getString(this, "fullname") + col, false, in, out));
                getValue = true;
            }

            //Indicate current value for editting
            if (edit)
            {
                out.println(BDialect.getString(this, "email") + col + user.getEmailAddress());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue)
            {
                //Get the email address
                user.setEmailAddress(clTools.doGetString(BDialect.getString(this, "email") + col, false, in, out));
                getValue = true;
            }

            //Indicate current value for editting
            if (edit)
            {
                out.println(BDialect.getString(this, "alias") + col + user.getAlias());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue)
            {
                //Get the alias
                user.setAlias(clTools.doGetString(BDialect.getString(this, "alias") + col, false, in, out));
                getValue = true;
            }


            //Indicate current value for editting
            if (edit)
            {
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "changepwq") + col, in, out));
            }

            //See if we still need to get the value
            if (getValue)
            {
                //Get the password
                user.setPassword(clTools.doGetString(BDialect.getString(this, "passwd")+ col, true, in, out));
                getValue = true;
            }

            //Indicate current value for editting
            if (edit && isAdmin)
            {
                out.println(BDialect.getString(this, "privilege") + col + this.clTools.getPrivilege(user));
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue && isAdmin)
            {
                //Get the privilege of the user
                user.setPrivilege(clTools.doGetInt(BDialect.formatText(BDialect.getString(this, "privilegeprompt"), new Object[]{new Integer(BPrivileged.kUser), new Integer(BPrivileged.kAdministrator)}), in, out, true, BPrivileged.kUser, BPrivileged.kAdministrator));
                getValue = true;
            }

            //Indicate current value for editting
            if (edit && isAdmin)
            {
                out.println(BDialect.getString(this, "groups") + col + user.getMembers());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue && isAdmin)
            {
                //Make sure we don't have conflict
                synchronized(this.availableGroups)
                {
                    //Get the groups of this new user
                    out.println(BDialect.getString(this, "selectgroups") + col);
                    user.setMembers(this.clTools.getMembers(this.availableGroups, in, out));
                    getValue = true;
                }
            }

            //Indicate current value for editting
            if (edit)
            {
                out.println(BDialect.getString(this, "comments") + col + user.getComments());
                getValue = (this.clTools.doConfirm(BDialect.getString(this, "editq"), in, out));
            }

            //See if we still need to get the value
            if (getValue)
            {
                //Get the comments
                user.setComments(clTools.doGetString(BDialect.getString(this, "comments") + col, false, in, out));
            }
        }
        catch(BInvalidPropertyException invalidException)
        {
        }
    }



    /**
     * When the user types "help" at the command prompt, the kernel
     * will call the getHelp method on all the registered onCommand
     * objects. This will indicate how to use this command line option.
     *
     * @param   params  The paramatesr to the help call (eg. help nadduser)
     */
      public String getHelp( BStringList params )
      {
        return BDialect.getString(this, "userclhelp");
      }
   }
