

package au.edu.swin.synergy.plugin.userman;

import au.edu.swin.synergy.plugin.command.BOnCommand;
import au.edu.swin.synergy.lib.BInputReader;

import au.edu.swin.synergy.plugin.userdb.BUser;
import au.edu.swin.synergy.plugin.userdb.BGroup;
import au.edu.swin.synergy.plugin.userdb.BPrivileged;
import au.edu.swin.synergy.plugin.userdb.event.BAddUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BUpdateUserCE;
import au.edu.swin.synergy.plugin.userdb.event.BRemoveUserCE;
import au.edu.swin.synergy.plugin.userdb.BInvalidPropertyException;

import au.edu.swin.synergy.plugin.dialect.BDialect;

import au.edu.swin.synergy.kernel.BPlugin;

import au.edu.swin.synergy.lib.BStringList;
import au.edu.swin.synergy.plugin.command.event.BStandardOutCE;

import au.edu.swin.synergy.debug.Debug;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;

import java.util.Vector;

/**
 * This provides command tools for reading
 * user and group information from the command line
 *
 * @author Tristan Austin
 */
public class BUserManCLTools
{
    /**
     * The user manager on the client
     */
    private BUserMan userManager;

    /**
     * Indicates whether this has been initialised or not
     */
    private boolean isInitialised = false;

    /**
     * Creates a new tools object with the
     * user maanger to communicate with.
     *
     * @param   userManager The user manager to query
     */
    public BUserManCLTools(BUserMan userManager)
    {
        this.userManager = userManager;
    }

    /**
     * This will run through and display the names
     * of all the BPrivileged items in the given Vector
     *
     * @param   items   A list of BPrivileged items to be displayed
     */
    protected void doShowAll(Vector items, PrintStream out)
    {
        //Run through them all
        for (int i = 0; i < items.size(); i++)
        {
            out.println(items.elementAt(i));
        }
    }

    /**
     * This will find the privileged item in the given list
     * with the name provided and return it.
     *
     * @param   name    The name of the item to return
     * @param   list    The list to search through
     * @return  The privileged item found in the list
     */
    protected BPrivileged getPrivileged(String name, Vector list)
    {
        BPrivileged item;

        //Search the list
        for (int i = 0; i < list.size(); i++)
        {
            item = (BPrivileged)list.elementAt(i);

            //If the names match, return the item
            if (item.getName().equals(name))
            {
                return item;
            }
        }

        //Only gets here if nothing found
        return null;
    }

    /**
     * This will make sure that the users and groups
     * have been set initially. If it has already it
     * will do nothing, otherwise it will tell the user
     * manager to refresh the lists.
     */
    protected void doInitialisation()
    {
        //Only need to do this once
        if (!this.isInitialised)
        {
            //Do the refersh
            this.userManager.doRefresh();
        }
    }

    /**
     * Allows the other plugins to indicate
     * when they have been initialised.
     *
     * @param   initialised True to indicate initialised, false otherwise
     */
    protected void setInitialised(boolean initialised)
    {
        this.isInitialised = initialised;
    }

    /**
     * This method will prompt the user to select the groups
     * this new user should be a member of by displaying the
     * options and reading in the users responses.
     *
     * @param   members The available members to select from
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @returns A Vector of Strings representing the names of the available groups
     */
    protected Vector getMembers(Vector members, InputStream in, PrintStream out) throws IOException
    {
        Vector selectedMembers = new Vector();
        int memberCount = members.size();
        BPrivileged item;
        int response = 0;

        //Make sure there are members to choose from
        if (memberCount > 0)
        {
            //Display the available options
            for (int i = 0; i < members.size(); i++)
            {
                out.println(i + " - " + members.elementAt(i));
            }

            //Get responses until the user puts in -1
            while(response >= 0)
            {
                //Get the response from the user
                response = this.doGetInt(BDialect.getString(this, "selectionprompt") + " ", in, out, false, 0, (memberCount-1));

                //Make sure it's not an exit result
                if (response >= 0)
                {
                    //Map the numbers to members
                    item = (BPrivileged)members.elementAt(response);

                    //Make sure there are no duplicates
                    if (!selectedMembers.contains(item.getName()))
                    {
                        selectedMembers.addElement(item.getName());
                    }

                    //Indicate the currently selected members
                    out.println(selectedMembers);
                }
            }
        }
        else
        {
            //Indicate the fact that there are no members available
            out.println(BDialect.getString(this, "nomembers"));
        }

        //Return the members selected
        return selectedMembers;
    }

    /**
     * This will prompt the user with the given
     * message and return their response. The response
     * is defined as everything they type up until they
     * press the enter key.
     * <p>
     * If the user presses enter without giving a value,
     * it will re-prompt them
     *
     * @param   prompt
     */
    protected String doGetString(String prompt, boolean nonNull, InputStream in, PrintStream out) throws IOException
    {
        BInputReader reader = new BInputReader(in);
        String currentLine;

        do
        {
            out.print(prompt);
            out.flush();
            currentLine = reader.readLine();
            out.println(currentLine);
        }while(nonNull && currentLine.trim().length() == 0);

        //Return the line entered
        return currentLine;
    }

    /**
     * This will read an integer from user and make sure that the
     * value they type is a valid integer. It will re-prompt until they
     * enter a valid argument.
     *
     * @param   prompt  The text to display to the user
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @param   nonNull Indicates whether the response can be nothing
     * @param   lowerLimit  The minimum value the user can enter
     * @param   upperLimit  The maximum value the user can enter
     * @return The value the user enters or -1 if they enter nothing
     */
    protected int doGetInt(String prompt, InputStream in, PrintStream out, boolean nonNull, int lowerLimit, int upperLimit) throws IOException
    {
        boolean validNumber = false;
        String currentLine;
        int value = -1;

        //Keep going until we get something good
        while(!validNumber)
        {
            try
            {
                //Get whatever they type in
                currentLine = this.doGetString(prompt, nonNull, in, out);

                //Determine whether they entered a value or not
                if (currentLine.trim().length() != 0)
                {
                    value = Integer.valueOf(currentLine).intValue();

                    //Make sure the value lies within the range specified
                    if (value >= lowerLimit && value <= upperLimit)
                    {
                        validNumber = true;
                    }
                    else
                    {
                        out.println(BDialect.formatText(BDialect.getString(this, "valuerange"), new Object[]{new Integer(lowerLimit), new Integer(upperLimit)}));
                    }
                }
                else
                {
                    //Indicate user entered no value
                    value = -1;
                    validNumber = true;
                }
            }
            catch(NumberFormatException numberFormatException)
            {
                //Indicate failure
                out.println(BDialect.getString(this, "invalidnum"));
            }
        }

        //Return the value
        return value;
    }

    /**
     * This will prompt the user for confirmation as to whether
     * they wish to go through with the current operation and
     * return true or false depending on their response.
     *
     * @param   in  The input stream to read from
     * @param   out The output stream to write to
     * @return True to confirm, false to deny
     */
    protected boolean doConfirm(String question, InputStream in, PrintStream out) throws IOException
    {
        String response;
        String affirmative = BDialect.getString(this, "y");
        String deny = BDialect.getString(this, "n");

        //Keep prompting until we get a y or n
        do
        {
            response = this.doGetString(question + " (" + affirmative + "/" + deny + "): ", true, in, out);
        }while(!response.equalsIgnoreCase(affirmative) && !response.equalsIgnoreCase(deny));

        //Value is either y or n, return true if y, false otherwise
        return (response.equalsIgnoreCase(affirmative));
    }

    /**
       * Provides a means of sending a message to standard out
       * in a common place rather than re-writing everything.
       *
       * @param message The message to send
       * @param messageType The message type BStandardOutCE.kWarning, BStandardOutCE.kError, BStandardOutCE.kInfo, BStandardOutCE.kDefaultOption
       */
      protected void doMessageSend(String message, int messageType)
      {
        //The user is not logged in, indicate this and do nothing
        BStandardOutCE standardOutCE = new BStandardOutCE(this, message);
        standardOutCE.setMessageType(messageType);

        //Dispatch the event
        this.userManager.dispatchEvent(standardOutCE);
      }

      /**
       * This method will check to see if the currently
       * logged in user is an administrator and if not
       * display a message indicating that they do
       * not have the required permission to perform the current
       * operation.
       *
       * @param out Indicates whether to display an error message or not
       * @return True if they are, false otherwise
       */
      protected boolean doAdminCheck(boolean displayMessage)
      {
        boolean result = this.userManager.isAdministrator();

        //Determine whether to display the message
        if (!result && displayMessage)
        {
            this.doMessageSend(BDialect.getString(this, "needadmin"), BStandardOutCE.kError);
        }

        //Return the result
        return result;
      }

      /**
       * This method will check to see whether the current user
       * is the owner of the given user name
       *
       * @param username    The user to check
       * @param displayMessage  Indicates whether or not to display the error
       * @return True if they are the owner, false otherwise
       */
      protected boolean doOwnerCheck(String userName, boolean displayMessage)
      {
        boolean owner = this.userManager.getCurrentUser().getName().equals(userName);

        //Determine whether to display the message
        if (!owner && displayMessage)
        {
            this.doMessageSend(BDialect.getString(this, "needadmin"), BStandardOutCE.kError);
        }

        return owner;
      }

      /**
       * This will get the privilege from the given
       * privileged item and return a String representing
       * it in human readable format.
       *
       * @param item    The privileged item to get the privilege of
       * @return The privilege as a String
       */
      protected String getPrivilege(BPrivileged item)
      {
        int privilege = item.getPrivilege();
        String result = null;

        //Determine the privilege
        if (privilege == BPrivileged.kAdministrator)
        {
            result = BDialect.getString(this, "admindisplay");
        }
        else if (privilege == BPrivileged.kUser)
        {
            result = BDialect.getString(this, "userdisplay");
        }

        //Return the result
        return result;
      }
}






