/**
 * Abstract superclass for all primitive shapes that
 * require rectangular regions as their bounds
 * @author Imron Alston
 * @copyright team synergy 1999
 */
//$Header: /teamproj/cvsroot/teamb/bkasrc/au/edu/swin/synergy/shapes/BRectShape.java,v 1.47 1999/10/15 09:57:55 tp079511 Exp $

package au.edu.swin.synergy.shapes;

import java.awt.*;
import java.awt.geom.*;
import java.util.Vector;
import java.lang.Math;

import au.edu.swin.synergy.plugin.diagram.BDrawable;
import au.edu.swin.synergy.shapes.BRectShapeProperties;
import au.edu.swin.synergy.shapes.BPropPanel;
import au.edu.swin.synergy.debug.Debug;
import au.edu.swin.synergy.lib.io.BSaveable;
import au.edu.swin.synergy.lib.io.BDataFile;
import au.edu.swin.synergy.lib.io.BStrokeIO;
import au.edu.swin.synergy.lib.BColorCombo;

import au.edu.swin.synergy.plugin.diagram.BView;
import au.edu.swin.synergy.math.BScalarRect;
import au.edu.swin.synergy.math.BScalar;
import au.edu.swin.synergy.math.BScalarPoint;


public abstract class BRectShape extends BDrawable implements BSaveable
{
	private static final String kDefaultName = "badname";

	private static final int kMinW = 5;
	private static final int kMinH = 5;

	protected BScalarRect local = null;
	private BScalarRect view = null;


	protected Rectangle xformed = null;
	protected Rectangle tmpXformed = null;

	protected boolean doFill = true;
	protected boolean doOutline = true;
	protected boolean useFillColour = true;
	protected boolean useOutlineColour = true;

	protected Color fillColour = Color.white;
	protected Color outlineColour = Color.black;
    protected Color lineColour = Color.black;

	protected int sizerNum = kNotHit;

	/**
	* When multiple lines are drawn, this is assigned line width,
	* instead creating new instance.
	*/
	protected int lineSize;

	protected Integer shapeStyle = BLineEditorPanel.kSingle;

	transient protected BasicStroke stroke = null;


	/**
	 * Renders the transformed coordinates of the drawable
	 * object to a Graphics2D
	 * Left to subclasses to implement as needed
	 * Subclasses should check to make sure if they are supposed
	 * to fill or outline the shape, as well as whether or not
	 * they should use their own colours.
	 *
	 * @param Graphics2D g2 - the place to render the object to.
	 */
	public void renderRectShape( Graphics2D g2 )
	{
		// Bounding box and others rely on xform after rendering.  Keep tmp
		// value of this, so it may be reset once rendering completed.
		this.tmpXformed.setBounds ( this.xformed );

		this.lineSize = ( int ) Math.round ( stroke.getLineWidth () );

		// Depending on the style of this shape, we should...

		this.renderRect ( g2, shapeStyle.intValue () );
	}

	public abstract void renderFill ( Graphics2D g2 );

	public abstract void renderShape ( Graphics2D g2 );

	public void renderRect ( Graphics2D g2, int noLines )
	{
		if ( noLines == BLineEditorPanel.kSingle.intValue () || noLines == BLineEditorPanel.kNone.intValue () )
		{
			if ( this.doFill )
			{
				this.renderFill ( g2 );
			}
			this.renderShape ( g2 );
			this.xformed.setBounds ( this.tmpXformed );
		}
		else
		{
			this.renderShape ( g2 );
			xformed.grow ( -this.lineSize * 2, -this.lineSize * 2 );
			renderRect ( g2, noLines - 1 );
		}
	}
	

	/**
	 * Creates and returns a new instance of the BDrawable object
	 *
	 * @return a new instance of this BDrawable object.
	 */
	public abstract BDrawable newInstance();


	protected BRectShape( int x, int y, int w, int h )
	{
		super();

		this.local = new BScalarRect( x, y, w, h );
		this.view = new BScalarRect();
		this.xformed = new Rectangle();
		this.tmpXformed = new Rectangle ();

		this.updateBounds();
	}

	protected BRectShape()
	{
		this( 0, 0, 0, 0 );
	}



	/**
	 * Transforms the coordinates of the rectshape
	 * to a new coordinate space, as defined by the
	 * viewport and parent pos.
	 *
	 * @param BView viewport - the viewport that this object is going to be drawn in
	 * @param Point parentPos - coordinates of any parent object
	 */
	public void transform( BView viewport, BScalarPoint parentPos, int frameCounter )
	{
		BScalar zoom = viewport.getOOZ();

		if ( this.lastFrame != frameCounter )
		{
			this.view.setBounds( this.local );
			this.view.incXY( this.pos );
			this.view.incXY( parentPos );
			this.view.decXY( viewport.getViewCoords() );
			this.view.scale( zoom );
			this.view.incXY( viewport.getHalfWH() );
			this.view.getBounds( this.xformed );

			this.lastFrame = frameCounter;
		}
	}

	public void render( Graphics2D g2 )
	{
		this.renderRectShape( g2 );
		if ( this.isSelected() )
		{
			this.renderOutlineAndSizers( g2 );
		}
	}



	/**
	 * Only render the outline and the sizing boxes....
	 *
	 */
	public void renderOutlineAndSizers( Graphics2D g2 )
	{
		int x, y, w, h;

		x = this.xformed.x - kSizerHalfW;
		y = this.xformed.y - kSizerHalfH;
		w = kSizerHalfW * 2;
		h = kSizerHalfH * 2;

		g2.setColor( Color.green );

		g2.fillRect( x, y, w, h );

		x += this.xformed.width;
		g2.fillRect( x, y, w, h );

		y += this.xformed.height;
		g2.fillRect( x, y, w, h );

		x -= this.xformed.width;
		g2.fillRect( x, y, w, h );
	}


	public void paintIcon( Component c, Graphics g, int x, int y )
	{
		this.xformed.x = x - c.getWidth() / 4;
		this.xformed.y = y - c.getHeight() / 4;
		this.xformed.width = c.getWidth() / 2;
		this.xformed.height = c.getHeight() / 2;

		render( (Graphics2D)g );
	}

	/**
	 * Checks to see if a point has hit the transformed bounds
	 * of this object
	 * @param Point pt - the point to check the hit against..
	 * @return kNotHit - not hit
	 * @return kHitBounds - hit the main bounds
	 */
	public int checkXFormBounds( Point pt )
	{
		return this.xformed.contains( pt ) ? kHitBounds : kNotHit;
	}


	/**
	 * Checks to see if a sizer has been hit by a point
	 * Works on xformed values.
	 * Subclasses should determine appropriate places for
	 * the sizer..
	 *
	 * @param Point pt - the point to check the hit against
	 * @return kNotHit - not hit
	 * @return kHitSizer + XX hit the XX sizer where XX is any number.
	 */
	public int checkSizerHit( Point pt )
	{
		int x1, y1, x2, y2;

		x1 = this.xformed.x - kSizerHalfW;
		y1 = this.xformed.y - kSizerHalfH;
		x2 = x1 + kSizerHalfW * 2;
		y2 = y1 + kSizerHalfH * 2;


		if ( pt.x > x1 && pt.x < x2 && pt.y > y1 && pt.y < y2 ) return kSizerNorthWest;

		x1 += this.xformed.width;
		x2 += this.xformed.width;

		if ( pt.x > x1 && pt.x < x2 && pt.y > y1 && pt.y < y2 ) return kSizerNorthEast;

		y1 += this.xformed.height;
		y2 += this.xformed.height;

		if ( pt.x > x1 && pt.x < x2 && pt.y > y1 && pt.y < y2 ) return kSizerSouthEast;

		x1 -= this.xformed.width;
		x2 -= this.xformed.width;

		if ( pt.x > x1 && pt.x < x2 && pt.y > y1 && pt.y < y2 ) return kSizerSouthWest;

		return kNotHit;
	}

	public void startAdd()
	{
		this.local.setBounds(0, 0,0, 0);
		this.sizerNum = kSizerSouthEast;
	}

	public void dragAdd( BScalarPoint deltaXY )
	{
		dragResize( deltaXY );
	}

	public void stopAdd()
	{
		this.local.setMinimumWH( kMinW, kMinH );
		this.sizerNum = kNotHit;
		this.updateBounds();
	}


	/**
	 * called when the drawable is about to be resized..
	 *
	 * @param int sizerNum - a sizerNum returned from checkSizerHit
	 */
	public void startResize( int sizerNum )
	{
		this.sizerNum = sizerNum;
	}

	/**
	 * Called as the drawable is being resized
	 *
	 */
	public void dragResize( BScalarPoint deltaXY )
	{
		//size it...
		switch ( this.sizerNum )
		{
			case kSizerNorthWest:
				this.local.decWH( deltaXY );
				this.pos.incXY( deltaXY );
				break;
			case kSizerNorthEast:
				this.local.incWDecH( deltaXY );
				this.pos.incY( deltaXY );
				break;
			case kSizerSouthEast:
				this.local.incWH( deltaXY );
				break;
			case kSizerSouthWest:
				this.local.decWIncH( deltaXY );
				this.pos.incX( deltaXY );
				break;
		}
	}

	public void stopResize()
	{
		this.sizerNum = kNotHit;
		this.updateBounds();
	}


	/**
	 * updates the bounding region of the drawable
	 * Called when something requires the bounds of
	 * the drawable to be updated.  Subclasses should
	 * determine a suitable way to update themselves.
	 */
	public void updateBounds()
	{
		this.bounds.setBounds( this.local );		
	}



	/**
	 * copies the data from the src BRectShape into this one.
	 * @param BRectShape src - the sourc BRectShape..
	 */
	public void copy( BRectShape src )
	{
		super.copy( src );

		this.local.setBounds( src.local );
		this.view.setBounds( src.view );
		this.xformed.setBounds( src.xformed );

		this.doFill = src.doFill;
		this.doOutline = src.doOutline;
		this.useFillColour = src.useFillColour;
		this.useOutlineColour = src.useOutlineColour;

		this.fillColour = src.fillColour;
		this.outlineColour = src.outlineColour;
        this.lineColour = src.lineColour;

		this.stroke = new BasicStroke( src.stroke.getLineWidth(),
									 src.stroke.getEndCap(),
									 src.stroke.getLineJoin(),
									 src.stroke.getMiterLimit(),
									 src.stroke.getDashArray(),
									 src.stroke.getDashPhase() );

		this.shapeStyle = src.shapeStyle;
	}

    public BPropPanel getPropertyPanel ()
    {
        if ( this == null )
        {
            Debug.doPrintln ( "BRectShape don't exist..." );
        }
        Debug.doPrintln ( "getPropPanel called..." );

        return new BRectShapeProperties ( this );
    }


	public void setLocalXY( BScalarPoint pt )
	{
		this.local.setXY( pt );
		this.bounds.setXY( pt );
	}

	public void setLocalWH( int w, int h )
	{
		this.local.setWH( w, h );
		this.bounds.setWH( w, h );

	}

	public void setLocalBounds ( BRectShape rec )
	{
		this.local.setBounds ( rec.local );
	}

	public void setDoFill( boolean doFill )
	{
		this.doFill = doFill;
	}

    public void setFillColour ( Color colour )
    {		
        this.fillColour = colour;
		if ( colour == BColorCombo.kNoColour )
		{
			this.doFill = false;
		}
		else
		{
			this.doFill = true;
		}
    }

    public Color getFillColour ()
    {
        return this.fillColour;
    }

    public void setOutlineColour ( Color colour )
    {
        this.outlineColour = colour;
    }

	public void setLineColour( Color colour )
	{
		this.lineColour = colour;
	}

    public Color getLineColour ()
    {
        return this.lineColour;
    }

	public void setStroke( BasicStroke stroke )
	{
		this.stroke = stroke;
	}

	public void setStyle ( Integer style )
	{
		this.shapeStyle = style;
		this.doOutline = style.intValue () != BLineEditorPanel.kNone.intValue ();
	}

	public Integer getStyle ()
	{
		return this.shapeStyle;
	}

    public BasicStroke getStroke ()
    {
        return this.stroke;
    }

	public void setHighlighted( boolean on )
	{
		if ( !this.isSelected() )
		{
			super.setHighlighted( on );

			if ( this.isHighlighted() )
			{
				this.outlineColour = Color.blue;
			}
			else
			{
				this.outlineColour = lineColour;
			}
		}
	}

	public void setSelected( boolean on )
	{
		super.setSelected( on );

		if ( this.isSelected() )
		{
			this.outlineColour = Color.green;
		}
		else
		{
			this.outlineColour = lineColour;
		}
	}

	/**
	 * loads a BRectShape form a data file
	 *
	 * @param file the file to load the BRectShape from
	 * @returns the success of the load
	 */

 	// for BSaveable
	protected static final String kKeyDoFill = "doFill";
	protected static final String kDefaultDoFill = "true";

	protected static final String kKeyUseFillColor = "useFillColour";
	protected static final String kDefaultUseFillColor = "true";

	protected static final String kKeyFillColor = "fillColour";
	protected static final String kDefaultFillColor = "" + Color.white.getRGB();

	protected static final String kKeyDoOutline = "doOutline";
	protected static final String kDefaultDoOutline = "true";

	protected static final String kKeyUseOutlineColor = "useOutlineColour";
	protected static final String kDefaultUseOutlineColor = "true";

	protected static final String kKeyOutlineColor = "outlineColour";
	protected static final String kDefaultOutlineColor = "" + Color.black.getRGB();

	protected static final String kKeyShapeStyle = "shapeStyle";
	protected static final String kDefaultShapeStyle = "1";

	protected static final String kKeyLineColor = "lineColour";
	protected static final String kDefaultLineColor = "" + Color.black.getRGB();

	protected static final String kKeyStrokeWidth = "width";
	protected static final String kDefaultStrokeWidth = "1.0";

	protected static final String kKeyStrokeCap = "cap";
	protected static final String kDefaultStrokeCap = "" + BasicStroke.CAP_SQUARE;

	protected static final String kKeyStrokeJoin = "join";
	protected static final String kDefaultStrokeJoin = "" + BasicStroke.JOIN_MITER;

	protected static final String kKeyStrokeLimit = "limit";
	protected static final String kDefaultStrokeLimit = "1.0";

	protected static final String kKeyStrokePhase = "phase";
	protected static final String kDefaultStrokePhase = "0.0";

	protected static final String kKeyStrokeDash = "dash";

	public boolean load( BDataFile file )
	{
		Debug.doPrintln("BRectShape load");
		super.load(file);
		doFill = new Boolean( file.getData( kKeyDoFill, kDefaultDoFill) ).booleanValue();
		useFillColour = new Boolean( file.getData( kKeyUseFillColor, kDefaultUseFillColor ) ).booleanValue();
		fillColour = new Color( new Integer( file.getData( kKeyFillColor, kDefaultFillColor ) ).intValue() );

		doOutline = new Boolean( file.getData( kKeyDoOutline, kDefaultDoOutline ) ).booleanValue();
		useOutlineColour = new Boolean( file.getData( kKeyUseOutlineColor, kDefaultUseOutlineColor ) ).booleanValue();
		outlineColour = new Color( new Integer( file.getData( kKeyOutlineColor, kDefaultOutlineColor ) ).intValue() );
		lineColour = new Color( new Integer( file.getData( kKeyLineColor, kDefaultLineColor ) ).intValue() );

		shapeStyle = Integer.valueOf ( file.getData ( kKeyShapeStyle, kDefaultShapeStyle ) );
		Debug.doPrintln ( "shape style : " + shapeStyle.intValue () );

		// load the stroke bits...
		stroke = BStrokeIO.load ( file );


		local = (BScalarRect) file.autoLoad();
		return true;
	}

	/**
	 * saaves the BRectShape to a data file
	 *
	 * @param file the file to save the rect shape to
	 * @returns the success of the save
	 */
	 public boolean save( BDataFile file )
	 {
		Debug.doPrintln( "BRectShape save" );
		super.save( file );

		file.addData( kKeyDoFill, new Boolean(doFill).toString(), kDefaultDoFill );
		file.addData( kKeyUseFillColor, new Boolean(useFillColour).toString(), kDefaultUseFillColor );
		file.addData( kKeyFillColor, Integer.toString( fillColour.getRGB() ), kDefaultFillColor );

		file.addData( kKeyDoOutline, new Boolean(doOutline).toString(), kDefaultDoOutline );
		file.addData( kKeyUseOutlineColor, new Boolean(useOutlineColour).toString(), kDefaultUseOutlineColor );
		file.addData( kKeyOutlineColor, Integer.toString( outlineColour.getRGB() ), kDefaultOutlineColor );

		file.addData ( kKeyShapeStyle, shapeStyle.toString (), kDefaultShapeStyle );
		
		// the stroke...
		BStrokeIO.save ( file, stroke );		

		file.save((BSaveable) local);
		return true;
	 }

}
