//$Header: /teamproj/cvsroot/teamb/debugsrc/au/edu/swin/synergy/debug/Strip.java,v 1.1 1999/08/08 02:31:54 tp079519 Exp $

package au.edu.swin.synergy.debug;

import java.io.*;


/**
 * The strip application will act as a pre-processor to the compiler, it will
 * run through the source files in the specified
 * directory and (if requested) recursively comment out or uncomment the debug
 * information in the code.
 * <p>
 * This class also supports specific additional tags the developer can include in
 * the source code to designate a block of code, spanning multiple lines, as a debug
 * block which can be toggled on and off (commented) at the users whim.
 * <p>
 * <B>Usage</B>
 * <p>
 * When run from the command line, it should be supplied with the starting directory
 * to recurse through as well as a flag indicating whether to strip or revive the
 * debugging information.
 * <p>
 * To specify the directory, use the -d <i>directoryname</i> option
 * <p>
 * To specify the type of run (strip or revive) use <b>-s</b> to strip and <b>-r</b> to revive
 * <p>
 * When using the debug classes in the code, you can comment out lines of code within
 * the debug block or just a debug or log class reference itself. When the Strip application
 * runs over this, it will not prepend another set of comments on the line so that when you
 * go to strip these comments out you only need to run over it once and your original comments remain.
 * <p>
 * It should be noted that if you run the Strip in revive mode over a class that has
 * not been stripped, any debug information that has been commented out by the coder
 * may be revived as well.
 * <p>
 * To ensure that this doesn't happen, coder inserted comments should be placed next
 * to the statement to comment out rather than against the left hand margin.
 * <p><blockquote><pre>
 * <b>eg.</b>
 * <code>//          Debug.doFatalError("Just kidding");</code>         <b>ill-advised</b>
 * <code>            //Debug.doFatalError("Just kidding");</code>       <b>recommended</b>
 * </blockquote></pre>
 * @author Tristan Austin
 */
public class Strip
{
	/**
	 * The starting directory
	 */
	private static String startingDir = ".";

    /**
     * The inserted tag for commenting out lines
     */
    private static String kLineComment = "//@db";

	/**
	 * The type of functionality to perform
	 */
	private boolean strip = false;

   /**
    * This indicates whether we are in a comment block or an active block
    */
   private boolean activeState = true;

	/**
	 * When the Strip application is run, this method is called. Any arguments
	 * given on the command line, or as paramaters to the program in any way,
	 * will be supplied in the String array. The paramaters required are defined
	 * by the program as it is run.
	 *
	 * @param	args	The command line arguments
	 */
	public static void main(String args[])
	{
		Strip stripper = new Strip();
		boolean strip = false;
      boolean typeSpecified = false;
      boolean argumentsValid = true;

      //Initialise the debugging classes
      Debug.doInitialisation();

		//See if we have the arguments
		if (args.length == 0)
		{
			argumentsValid = false;
		}
		else if (args.length > 0)
		{
         for (int i = 0; i < args.length; i++)
         {
            //See whether it is strip
            if (args[i].equalsIgnoreCase("-s"))
            {
               if (!typeSpecified)
               {
                  //Need to strip debug info
                  Debug.doPrintln("Strip: Stripping debug information.");
                  strip = true;
                  typeSpecified = true;
               }
               else
               {
                  System.out.println("You can only specify one of -s or -r\n");
                  typeSpecified = false;
                  argumentsValid = false;
               }
            }
            else if (args[i].equalsIgnoreCase("-r"))
            {
               if (!typeSpecified)
               {
                  //Need to uncomment debug info
                  Debug.doPrintln("Strip: Reviving debug information.");
                  strip = false;
                  typeSpecified = true;
               }
               else
               {
                  System.out.println("You can only specify one of -s or -r\n");
                  typeSpecified = false;
                  argumentsValid = false;
               }
            }
            else if (args[i].equalsIgnoreCase("-d"))
            {
               //The next argument should specify the directory
               if (i +1 < args.length)
               {
                  //Initialise the starting directory
                  Strip.startingDir = args[++i];
               }
               else
               {
                  //Set valid to false
                  argumentsValid = false;
               }
            }
         }
		}

      //Make sure its ok to continue
      if (argumentsValid)
      {
         //Do the work
         stripper.setStrip(strip);
         stripper.doDirRecurse(new File(Strip.startingDir));
      }
      else
      {
         Strip.doPrintUsage();
      }
	}

	/**
	 * Sets the functionality of the application to either stripping
	 * or reviving
	 *
	 * @param	strip	The new value of strip, true for stripping, false for reviving
	 */
	public void setStrip(boolean strip)
	{
		this.strip = strip;
	}

	/**
	 * When this method is called, it runs through the source files recurisvely from
	 * the directory the program was started in and comments out all the debug information.
	 */
	public void doDirRecurse(File file)
	{
		//Create a file representing the current directory
		String[] contents = null;
		String currentFile = null;

      //Check to see that it is a directory
      if (file.isDirectory())
      {
         //Get the files in this directory
         contents = file.list();

         //Run through contents of directory
         for (int i = 0; i < contents.length; i++)
         {
            currentFile = contents[i];
            Debug.doPrintln(file.getName() + ": " + currentFile);
            this.doDirRecurse(new File(file, currentFile));
         }
      }
      else
      {
         //Get the name of the file
         currentFile = file.getName();

         //check if the file is a .java file
         if (currentFile.endsWith(".java"))
         {
            //Run through the file
            this.doProcessing(file);
         }
      }
	}

	/**
	 * This method will comment out all the debuggin info from
	 * the given file.
	 *
	 * @param	file	The file to strip
	 */
	public void doProcessing(File file)
	{
      File outFile = null;
		FileOutputStream outputStream = null;
      FileInputStream inputStream = null;
      InputStreamReader streamReader = null;
		PrintWriter filePrinter = null;
      LineNumberReader inputFile = null;
		String strippedPath = file.getPath() + ".dbg";
		String currentLine = null;

		Debug.doPrintln("Stripping debug info from " + file.getName());

		try
		{
         //Create the file to write to
         outFile = new File(strippedPath);

			//Open output stream for stripped file
			outputStream = new FileOutputStream(outFile);

         //Open the input stream
         inputStream = new FileInputStream(file);

         //Create the input stream reader
         streamReader = new InputStreamReader(inputStream);

			//Create the stripped file
			filePrinter = new PrintWriter(outputStream, true);

			//Create the random access to the original file
         inputFile = new LineNumberReader(streamReader);

			//Read through the original file
         while((currentLine = inputFile.readLine()) != null)
			{
            //Determine if the given line is a single line of tagged code
            if (this.isSingleTaggedLine(currentLine))
            {
               //Determine whether to strip or revive
               if (this.strip)
               {
                  //Strip
                  this.doStripLine(currentLine, filePrinter);
               }
               else
               {
                  //Revive
                  this.doReviveLine(currentLine, filePrinter);
               }
            }
            else if (this.isStartDebug(currentLine))
            {
               //If we have hit the start of a debug block, keep commenting until
               //we find the end tag

               //Write the start tag to the output file for future reference
               filePrinter.println(currentLine);

               //Loop while not at end of file and not at end of debug block
               //while((currentLine = stripped.readLine()) != null && !this.isEndDebug(currentLine))
               while((currentLine = inputFile.readLine()) != null && !this.isEndDebug(currentLine))
               {
                  //Determine whether to strip or revive
                  if (this.strip)
                  {
                     //Strip
                     this.doStripLine(currentLine, filePrinter);
                  }
                  else
                  {
                     //Revive
                     this.doReviveLine(currentLine, filePrinter);
                  }
               }

               //Write the end tag to the output file
               if (currentLine != null)
               {
                  filePrinter.println(currentLine);
               }
            }
            else
            {
               //Write the line unchanged to the new file
               filePrinter.println(currentLine);
            }
			}

         //Close the files
			filePrinter.close();		//*/
         //stripped.close();
         inputFile.close();

         //Need to rename the files
			if (file.delete())
         {
            //Rename output file to match old file
            outFile.renameTo(file);
         }
         else
         {
            Debug.doPrintln("The stripped file: " + file.getName() + " could not be deleted.");
            Debug.doPrintln("The stripped version is in the file: " + outFile.getName());
         }
		}
		catch(Exception ioException)
		{
			ioException.printStackTrace();
		}
	}

   /**
    * Indicates whether the given line is the
    * start of a debugging block of code or not
    *
    * @param   line  The line to check
    * @returns True if it is, false otherwise
    */
   private boolean isEndDebug(String line)
   {
      return this.isTaggedLine(line, "//@db}");
   }

   /**
    * Indicates whether the given line is the
    * start of a debugging block of code or not
    *
    * @param   line  The line to check
    * @returns True if it is, false otherwise
    */
   private boolean isStartDebug(String line)
   {
      return this.isTaggedLine(line, "//@db{");
   }

   /**
    * This will indicate whether the given line is a debug
    * line which needs to be commented or uncommented.
    *
    * @param   line  The line to check
    * @returns True if it is, false otherwise
    */
   private boolean isSingleTaggedLine(String line)
   {
      boolean singleLine = false;

      //Check for given tags
      singleLine = (this.isTaggedLine(line, "Debug.")) || (this.isTaggedLine(line, "Log."));

      //Return result
      return singleLine;
   }

   /**
    * This will indicate whether the given line contains the
    * given tag.
    *
    * @param   line  The line to check
    * @returns True if it does, false otherwise
    */
   private boolean isTaggedLine(String line, String tag)
   {
      int knownIndex = -1;
      int startBlockIndex = -1;
      int endBlockIndex = -1;
      int lineCommentIndex = -1;
      boolean hasTag = false;
      char character = '\0';

      //Search for reserved values
      knownIndex = line.indexOf(tag);

      //If a reserved tag, comment it out
      hasTag = (knownIndex >= 0);

      //Make sure we're not in a comment block
      startBlockIndex = line.indexOf("/*");
      endBlockIndex = line.lastIndexOf("*/");
      lineCommentIndex = line.indexOf("//");

      //See if the line is already commented out
      if ((lineCommentIndex >= 0) && (this.strip))
      {
         //See if the next character is
         if (lineCommentIndex +2 < line.length())
         {
            if (line.charAt(lineCommentIndex +2) != '@')
            {
               //Doesn't matter if the comment is after the tag
               hasTag &= (lineCommentIndex > knownIndex);
            }
         }
         else
         {
            //Doesn't matter if the comment is after the tag
            hasTag &= (lineCommentIndex > knownIndex);
         }
      }

      //See if we're in the block statement
      if (startBlockIndex >= 0)
      {
         this.activeState = false;
         hasTag &= (startBlockIndex > knownIndex);
      }

      //See if we're at the end of the block comment
      if (endBlockIndex >= 0)
      {
         this.activeState = true;
         hasTag &= (endBlockIndex < knownIndex);
      }

      //Only recognise as a tag when stripping if it isn't commented out
      hasTag &= (this.activeState || !this.strip);

      //Make sure its not just a part of another word
      if (hasTag)
      {
         //Make sure we can check the character before the tag
         if (knownIndex > 0)
         {
            //Get the preceding character
            character = line.charAt(knownIndex -1);
         }

         //Make sure it isn't part of another word
         hasTag = ((!Character.isLetter(character) || Character.isWhitespace(character)) && character != '"');
      }

      //Return found
      return hasTag;
   }

   /**
    * This will add the comment symbol to the start of the given line
    * in the output file.
    *
    * @param   currentLine The line to comment out
    * @param   outputFile  The file to write to
    */
   private void doStripLine(String currentLine, PrintWriter outputFile)
   {
      //Comment out this line in the block
      outputFile.println((Strip.kLineComment + currentLine));
   }

   /**
    * This will revive the given line by uncommenting it, if and only
    * if it was commented out by the stripper, and write it to the
    * output print writer given
    *
    * @param   currentLine The line to revive
    * @param   output   The file to write to
    */
   private void doReviveLine(String currentLine, PrintWriter outputFile)
   {
      //See if the line was commented out by the stripper
      if (currentLine.startsWith(Strip.kLineComment))
      {
         //If so, write it to the new file without the comments
         outputFile.println(currentLine.substring(Strip.kLineComment.length(), currentLine.length()));
      }
      else
      {
         //Otherwise just write it as normal
         outputFile.println(currentLine);
      }
   }

   /**
    * Prints out the usage details of the Strip application
    */
   private static void doPrintUsage()
   {
      System.out.println("Strip usage: java Strip [-d startingDir] [-s][-r]");
      System.out.println("Where -s means strip debugging code,");
      System.out.println("-r means to revive the debugging code");
      System.out.println("and startingDir is the top level directory to start");
      System.out.println("in. Defaults to current directory if not specified.");
   }
}
