//$Header: /teamproj/cvsroot/teamb/nodebugsrc/au/edu/swin/synergy/debug/Log.java,v 1.1 1999/08/22 21:52:40 tp079519 Exp $

package au.edu.swin.synergy.debug;

import java.io.*;
import java.util.*;
import java.text.*;

/**
 * The log class will allow messages to be written to a log file. It provides
 * a means of appending or overwriting a file depending on the conents
 * of the properties file.
 * <p>
 * <p>
 * <B>Debug.prp</B>
 * <p>
 * There are three properties you can include in the Debug.prp properties file,
 * which is the same file used by the Debug class.
 * <p>
 * <b>overwrite = true</b>
 * <p>
 * This indicates whether to overwrite any previous log files or append the
 * new log information to the end of the last one. The latter can be useful given
 * that all logs are time stamped. If this property isn't supplied
 * it defaults to overwriting.
 * <p>
 * <b>logname = Debug01.log</b>
 * <p>
 * This indicates that the log file is called
 * Debug01.log (in this example). If this property isn't supplied, it
 * defaults to Debug.log
 * <p>
 * <b>logdir = logfiles</b>
 * <p>
 * This indicates the directory to put the log file in. In this case it uses a
 * directory caled logfiles. The default is the current directory (the directory
 * the program was run from.)
 * <p>
 * <b>messages = ErrorMessages.log</b>
 * <p>
 * This provides the name of the file to use when looking for pre-defined
 * error messages. The default is "Messages.log" if it is not specified here.
 * <p>
 * <b>Messages.log</b>
 * <P>
 * As stated above, the pre-defined error messages can be found in a file called
 * "Messages.log" or whatever is specified in the Debug.prp file. The messages
 * can be used to provide lengthy and detailed messages without having to write
 * them each time they are used.
 * <p>
 * For example: To define a message called "ioerror" in the file as being the name of the
 * message "An Input/Output error has ocurred, please shut-down and restart the system"
 * <p>
 * You would put in the messages.log (or the name specified) file:
 * <code>
 * ioerror = An Input/Output error has ocurred, please shut-down and restart the system
 * </code>
 * <p>
 * When the coder wishes to use this message, all they have to do is refer to it by
 * the "ioerror" name.
 * <p>
 * <b>eg.</b><code>
 * Log.doError("Short message", "ioerror", Debug.kErrorLevel);
 * </code>
 * <p>
 * The logged file will then include both the short message and the long message
 * referred to be ioerror.
 *
 * @author Tristan Austin
 */
public class Log
{
   /**
    * The name of the properties log file
    */
   private static String fileName = "Debug.log";

   /**
    * The directory to put it in
    */
   private static String fileDirectory = ".";

   /**
    * The name of the tag in the properties file
    */
   private static final String kOverwrite = "overwrite";

   /**
    * The value that should be entered into the properties file
    * to indicate true. Case is irrelevant.
    */
   private static final String kTrue = "true";

   /**
    * The prefix for error messages
    */
   private static final String kErrorPrefix = "ERROR: ";

   /**
    * The prefix for fatal error messages
    */
   private static final String kFatalPrefix = "FATALERROR: ";

   /**
    * The warning message prefix
    */
   private static final String kWarningPrefix = "Warning: ";

   /**
    * The constant property tag for the file name
    */
   private static final String kLogName = "logname";

   /**
    * The constant property tag for the file name
    */
   private static final String kLogDir = "logdir";

   /**
    * The file with the messages in it
    */
   private static String messagesFile = "Messages.log";

   /**
    * The constant property tag for the messages file name
    */
   private static final String kMessagesFile = "messages";

   /**
    * Indicates what error level to display
    */
   private static int errorLimit = 0;

   /**
    * The constant property tag for the error limit
    */
   private static final String kErrorLimit = "errorlimit";

   /**
    * This is where we keep the list of messages available to
    * the coders.
    */
   private static Properties messages = new Properties();

   /**
    * This indicates whether to overwrite the original log
    * file or append to it
    */
   private static boolean overwrite = false;

   /**
    * Indicates whether we have already overwritten the
    * first line
    */
   private static boolean overwritten = false;

   /**
    * This method will initialise the Debug class with
    * the defaults, or the text file provided.
    */
   public static void doInitialisation()
   {
      Properties properties = new Properties();
      FileInputStream fileIn = null;
      String property = null;

      //Initialise the Debug class if it hasn't been already
      if (!Debug.isInitialised())
      {
         Debug.doInitialisation();
      }

      try
      {
         //Open the input stream to the properties file
         fileIn = new FileInputStream(Debug.kPropertiesFileName);

         //Load the properties file
         properties.load(fileIn);

         //Get the user specified run level
         if (properties.containsKey(kOverwrite))
         {
            property = (String)properties.get(kOverwrite);

            //If the value isn't true, will default to false
            Log.overwrite = property.equalsIgnoreCase(kTrue);
         }

         //See if they have specified a file name for the log
         if (properties.containsKey(kLogName))
         {
            Log.fileName = (String)properties.get(kLogName);
         }

         //See if they have specified a directory for the log
         if (properties.containsKey(kLogDir))
         {
            Log.fileDirectory = (String)properties.get(kLogDir);
         }

         //See if they have specified an error level
         if (properties.containsKey(kErrorLimit))
         {
            property = (String)properties.get(kErrorLimit);

            Log.errorLimit = Integer.valueOf(property).intValue();
         }

         //Get the name of the messages file if specified
         if (properties.containsKey(kMessagesFile))
         {
            Log.messagesFile = (String)properties.get(kMessagesFile);
         }

         //Close the input stream
         fileIn.close();

         //Now we want to read in the messages
         fileIn = new FileInputStream(messagesFile);

         //Load the message properties
         Log.messages.load(fileIn);

         //Close the input stream
         fileIn.close();
      }
      catch(IOException ioException)
      {
         //Properties file probably doesn't exist
      }
   }

   /**
    * Places the given warning message into the log file if the
    * current debug level allows it.
    *
    * @param   warningMessage The warning message
    */
   public static void doPrintln(String message)
   {
      //Make sure we're in the right debug level
      if ( Debug.debugLevel <= Debug.kPrintLevel)
      {
         Log.doWriteln(message);
      }
   }

   /**
    * This will write the given error message to the log file along
    * with the error level. The error level is on par with the debug
    * levels used elsewhere. That is to say, you can have an error
    * of level Print, Warning, Error or Fatal. These are all
    * defined in the Debug class
    * <p>
    * This version allows the coder to specify the name of a predefined
    * message from the Messages.log file to be used as well.
    *
    * @param   message     The error message
    * @param   errorLevel  The error level
    */
   public static void doError(String message, String messageEntry, int errorLevel)
   {
      String newMessage = message;
      String tag = messageEntry.trim();

      //Add the predefined message if there is one
      if (Log.messages.containsKey(tag))
      {
         //Add the predefined message to the end of the first message
         newMessage += (String)Log.messages.get(tag);
      }

      //Call the original version with the new message
      Log.doError(newMessage, errorLevel);
   }

   /**
    * This will write the given error message to the log file along
    * with the error level. The error level is on par with the debug
    * levels used elsewhere. That is to say, you can have an error
    * of level Print, Warning, Error or Fatal. These are all
    * defined in the Debug class
    *
    * @param   message     The error message
    * @param   errorLevel  The error level
    */
   public static void doError(String message, int errorLevel)
   {
      //Make sure we're in the right debug level
      if ( (Debug.debugLevel <= Debug.kErrorLevel) && (Log.errorLimit <= errorLevel) )
      {
         Log.doWriteln(kErrorPrefix + "[" + errorLevel + "] " + message);
      }
   }

   /**
    * This will always write the fatal error message to the
    * log file and exit the application no matter what the
    * current run level is.
    *
    * @param   message  The message to log
    */
   public static void doFatalError(String message)
   {
      //Write the message to the log file
      Log.doWriteln(kFatalPrefix + message);

      //Exit the program
      System.exit(0);
   }

   /**
    * Places the given warning message into the log file if the
    * current debug level allows it.
    * <p>
    * This version allows the coder to specify a predefined message
    * in the Messages.log file by its title.
    *
    * @param   warningMessage The warning message
    */
   public static void doWarning(String warningMessage, String messageEntry)
   {
      String newMessage = warningMessage;
      String tag = messageEntry.trim();

      //Add the predefined message if there is one
      if (Log.messages.containsKey(tag))
      {
         //Add the predefined message to the end of the first message
         newMessage += (String)messages.get(tag);
      }

      //Call the original version with the new message
      Log.doWarning(newMessage);
   }

   /**
    * Places the given warning message into the log file if the
    * current debug level allows it.
    *
    * @param   warningMessage The warning message
    */
   public static void doWarning(String warningMessage)
   {
      //Make sure we're in the right debug level
      if ( Debug.debugLevel <= Debug.kWarningLevel)
      {
         doWriteln(kWarningPrefix + warningMessage);
      }
   }

   /**
    * This method will actually log the message in the log file. It
    * will also add the time stamp and the debug level of the message.
    *
    * @param   message     The message to log
    * @param   debugLevel  The debug level of the message
    */
   private static void doWriteln(String message)
   {
      File file = null;
      FileOutputStream fileOut = null;
      PrintWriter printWriter = null;
      String outputMessage = null;
      Date timeStamp = new Date();
      String timeStampString = Debug.dateFormatter.format(timeStamp);
      boolean appending = (!overwrite || overwritten);

      //Build the message
      outputMessage = message + " [timestamp: " + timeStampString + "]";

      try
      {
         //Create the file
         file = new File(fileDirectory, fileName);

         //Create the file outpuFileOutputStreamt stream with append set to true
         fileOut = new FileOutputStream(file.getAbsolutePath(), appending);

         //Flag that we have overwritten the original file
         overwritten = true;

         //Create the print writer based onthe file output stream
         printWriter = new PrintWriter(fileOut, true);

         //Writes the message to the file
         printWriter.println(outputMessage);

         //Close the output stream when done
         fileOut.close();
      }
      catch(IOException ioException)
      {
         //Problem writing to the file
         ioException.printStackTrace();
      }
   }
}